@@ -8,7 +8,7 @@ from base64 import b64encode
 
 from . import *  # noqa: F401, F403
 
-condor_accounting_group = 'ligo.prod.o3.cbc.pe.bayestar'
+condor_accounting_group = 'ligo.prod.o4.cbc.pe.bayestar'
 """HTCondor accounting group for Celery workers launched with condor_submit."""
 
 expose_to_public = True
@@ -21,6 +21,9 @@ igwn_alert_group = 'gracedb'
 gracedb_host = 'gracedb.ligo.org'
 """GraceDB host."""
 
+create_mattermost_channel = True
+"""Create Mattermost channel in production"""
+
 kafka_alert_config = {
     'scimma': {'url': 'kafka://kafka.scimma.org/igwn.gwalert',
                'suffix': 'avro', 'skymap_encoder': lambda _: _},
@@ -30,6 +33,16 @@ kafka_alert_config = {
 }
 """Kafka broker configuration details"""
 
+kafka_consumer_config = {
+    'fermi': {'url': 'kafka://kafka.gcn.nasa.gov/'
+              'fermi.gbm.targeted.private.igwn', 'suffix': 'json'},
+    'swift': {'url': 'kafka://kafka.gcn.nasa.gov/'
+              'gcn.notices.swift.bat.guano', 'suffix': 'json'}
+}
+"""Kafka consumer configuration details. The keys describe the senders of the
+messages to be consumed. The values are a dictionary of the URL to listen to
+and information about the message serializer."""
+
 voevent_broadcaster_address = ':5341'
 """The VOEvent broker will bind to this address to send GCNs.
 This should be a string of the form `host:port`. If `host` is empty,
@@ -58,9 +71,9 @@ high_latency_frame_types = {'H1': 'H1_HOFT_C00',
 checks (see :mod:`gwcelery.tasks.detchar`).
 """
 
-idq_channels = ['H1:IDQ-PGLITCH_OVL_16_4096',
-                'L1:IDQ-PGLITCH_OVL_16_4096']
-"""Low-latency iDQ p(glitch) channel names from live O3 frames"""
+idq_channels = ['H1:IDQ-FAP_OVL_10_2048',
+                'L1:IDQ-FAP_OVL_10_2048']
+"""Low-latency iDQ false alarm probability channel names from live O3 frames"""
 
 strain_channel_names = {'H1': 'H1:GDS-CALIB_STRAIN_CLEAN',
                         'L1': 'L1:GDS-CALIB_STRAIN_CLEAN',
@@ -77,11 +90,15 @@ only_alert_for_mdc = True
 """If True, then only sends alerts for MDC events. Useful for times outside
 of observing runs."""
 
-bilby_default_mode = 'production'
-"""Sampling mode of bilby"""
-
 condor_retry_kwargs = dict(
     max_retries=None, retry_backoff=True, retry_jitter=True,
     retry_backoff_max=600
 )
 """Retry settings of condor.submit task."""
+
+rapidpe_settings = {
+    'run_mode': 'online',
+    'accounting_group': 'ligo.prod.o4.cbc.pe.lalinferencerapid',
+    'use_cprofile': False
+}
+"""Config settings used for rapidpe"""
