@@ -21,22 +21,21 @@ import json
 import socket
 import time
 
+import matplotlib.pyplot as plt
+import numpy as np
+import sentry_sdk
 from astropy.time import Time
 from celery import group
 from celery.utils.log import get_task_logger
 from glue.lal import Cache
 from gwdatafind import find_urls
-from gwpy.timeseries import Bits, StateVector, TimeSeries
 from gwpy.plot import Plot
-import matplotlib.pyplot as plt
-import numpy as np
+from gwpy.timeseries import Bits, StateVector, TimeSeries
 
-from . import gracedb
-from ..import app
-from ..import _version
+from .. import _version, app
 from ..jinja import env
 from ..util import closing_figures
-
+from . import gracedb
 
 __author__ = 'Geoffrey Mo <geoffrey.mo@ligo.org>'
 
@@ -74,16 +73,19 @@ def create_cache(ifo, start, end):
     cache = Cache.from_urls(filenames)
 
     try:
-        cache_starttime = int(
-            list(cache.to_segmentlistdict().values())[0][0][0])
+        cache_segment = list(cache.to_segmentlistdict().values())[0][0]
+        cache_starttime = cache_segment[0]
+        cache_endtime = cache_segment[1]
+
     except IndexError:
         log.exception('Files do not exist in llhoft_glob')
         return cache  # returns empty cache
 
-    if start >= cache_starttime:  # required data is in llhoft
+    if (cache_starttime <= start) and (end <= cache_endtime):
+        # required data is in llhoft
         return cache
 
-    # otherwise, required data has left llhoft
+    # otherwise, required data is not in the low latency cache
     high_latency = app.conf['high_latency_frame_types'][ifo]
     urls = find_urls(ifo[0], high_latency, start, end)
     if urls:
@@ -110,9 +112,9 @@ def make_omegascan(ifo, t0, durs):
         'H1', 'L1', or 'V1'
     t0 : int or float
         Central time of the omegascan.
-    durs : list of floats/ints
-        List of three durations which will be scanned symmetrically about t0.
-        Example: [0.5, 2, 10]
+    durs : list of tuples
+        List of three tuples, with time before and time after t0 in seconds.
+        Example: [(0.75, 0.25), (1.5, 0.5), (7.5, 2.5)]
 
     Returns
     -------
@@ -124,8 +126,10 @@ def make_omegascan(ifo, t0, durs):
     plt.switch_backend('agg')
 
     # Collect data
-    longest = max(durs)
-    long_start, long_end = t0 - longest, t0 + longest
+    durs = np.array(durs)
+    longest_before, longest_after = durs[:, 0].max(), durs[:, 1].max()
+    # Add extra data buffer to avoid whitening window issues
+    long_start, long_end = t0 - longest_before - 2, t0 + longest_after + 2
     cache = create_cache(ifo, long_start, long_end)
     strain_name = app.conf['strain_channel_names'][ifo]
     try:
@@ -133,26 +137,36 @@ def make_omegascan(ifo, t0, durs):
                              start=long_start, end=long_end).astype('float64')
         # Do q_transforms for the different durations
         qgrams = [ts.q_transform(
-            frange=(20, 4096), gps=t0, outseg=(t0 - dur, t0 + dur), logf=True)
-            for dur in durs]
-    except (IndexError, FloatingPointError, ValueError):
+            frange=(10, 4096), gps=t0,
+            outseg=(t0 - before, t0 + after), logf=True)
+            for before, after in durs]
+    except (IndexError, FloatingPointError, ValueError) as err:
         # data from cache can't be properly read, or data is weird
-        fig = plt.figure()
+        sentry_sdk.capture_exception(err)
+        fig = plt.figure(figsize=(4, 1))
         plt.axis("off")
-        plt.text(0.1, 0.45, f"Failed to create {ifo} omegascan", fontsize=17)
+        plt.text(0.5, 0.5, f"Failed to create {ifo} omegascan",
+                 horizontalalignment='center', verticalalignment='center',
+                 fontsize=17)
     else:
         fig = Plot(*qgrams,
-                   figsize=(10 * len(durs), 5),
+                   figsize=(12 * len(durs), 6),
                    geometry=(1, len(durs)),
                    yscale='log',
-                   method='pcolormesh')
-        for ax in fig.axes:
-            fig.colorbar(ax=ax, label='Normalized energy', clim=(0, 30))
+                   method='pcolormesh',
+                   clim=(0, 30),
+                   cmap='viridis')
+        for i, ax in enumerate(fig.axes):
+            ax.set_title(f'Q = {qgrams[i].q:.3g}')
+            if i in [1, 2]:
+                ax.set_ylabel('')
+            if i == 2:
+                fig.colorbar(ax=ax, label='Normalized energy')
             ax.set_epoch(t0)
         fig.suptitle(f'Omegascans of {strain_name} at {t0}', fontweight="bold")
-
+    plt.subplots_adjust(wspace=0.08)
     outfile = io.BytesIO()
-    fig.savefig(outfile, format='png', dpi=300)
+    fig.savefig(outfile, format='png', dpi=150, bbox_inches='tight')
     return outfile.getvalue()
 
 
@@ -169,14 +183,16 @@ def omegascan(t0, graceid):
 
     """
     durs = app.conf['omegascan_durations']
+    durs = np.array(durs)
 
     # Delay for early warning events (ie queries for times before now)
-    if t0 + max(durs) > Time.now().gps:
+    longest_after = durs[:, 1].max() + 2  # extra data for whitening window
+    if t0 + longest_after > Time.now().gps:
         log.info("Delaying omegascan because %s is in the future",
                  graceid)
-        waittime = t0 - Time.now().gps + max(durs) + 10
+        waittime = t0 - Time.now().gps + longest_after + 10
     else:
-        waittime = max(durs) + 10
+        waittime = longest_after + 10
 
     group(
         make_omegascan.s(ifo, t0, durs).set(countdown=waittime)
@@ -258,6 +274,24 @@ def dqr_json(state, summary):
     )
 
 
+def ifo_from_channel(channel):
+    '''Get detector prefix from a channel.
+
+    Parameters
+    ----------
+    channel : str
+        Channel, e.g., H1:GDS-CALIB_STRAIN.
+
+    Returns
+    -------
+    str
+        Detector prefix, e.g., H1.
+    '''
+    ifo = channel.split(':')[0]
+    assert len(ifo) == 2, "Detector name should be two letters"
+    return ifo
+
+
 def check_idq(cache, channel, start, end):
     """Looks for iDQ frame and reads them.
 
@@ -266,31 +300,29 @@ def check_idq(cache, channel, start, end):
     cache : :class:`glue.lal.Cache`
         Cache from which to check.
     channel : str
-        which idq channel (pglitch)
+        which idq channel (FAP)
     start, end: int or float
         GPS start and end times desired.
 
     Returns
     -------
     tuple
-        Tuple mapping iDQ channel to its maximum P(glitch).
+        Tuple mapping iDQ channel to its minimum FAP.
 
     Example
     -------
-    >>> check_idq(cache, 'H1:IDQ-PGLITCH-OVL-100-1000',
+    >>> check_idq(cache, 'H1:IDQ-FAP_OVL_100_1000',
                   1216496260, 1216496262)
-    ('H1:IDQ-PGLITCH-OVL-100-1000', 0.87)
+    ('H1:IDQ-FAP_OVL_100_1000', 0.003)
 
     """
     if cache:
         try:
-            idq_prob = TimeSeries.read(
+            idq_fap = TimeSeries.read(
                 cache, channel, start=start, end=end)
-            return (channel, float(idq_prob.max().value))
-        except (IndexError, RuntimeError):
-            # FIXME: Change from log.exception to log.warning until this fixed,
-            # because it's saturating Sentry.
-            log.warning('Failed to read from low-latency iDQ frame files')
+            return (channel, float(idq_fap.min().value))
+        except (IndexError, RuntimeError, ValueError):
+            log.exception('Failed to read from low-latency iDQ frame files')
     # FIXME: figure out how to get access to low-latency frames outside
     # of the cluster. Until we figure that out, actual I/O errors have
     # to be non-fatal.
@@ -343,32 +375,27 @@ def check_vector(cache, channel, start, end, bits, logic_type='all'):
         try:
             statevector = StateVector.read(cache, channel,
                                            start=start, end=end, bits=bits)
-        except (IndexError, TypeError):
-            # FIXME: TypeError above is due to
-            # https://github.com/gwpy/gwpy/issues/1211
-            #
-            # FIXME: Change from log.exception to log.warning until this fixed,
-            # because it's saturating Sentry.
-            log.warning('Failed to read from low-latency frame files')
+        except (IndexError, TypeError, ValueError):
+            log.exception('Failed to read from low-latency frame files')
         else:
             # FIXME: In the playground environment, the Virgo state vector
             # channel is stored as a float. Is this also the case in the
             # production environment?
             statevector = statevector.astype(np.uint32)
             if len(statevector) > 0:  # statevector must not be empty
-                return {bitname.format(channel.split(':')[0], key):
+                return {bitname.format(ifo_from_channel(channel), key):
                         bool(logic_map[logic_type](
                             value.value if len(value.value) > 0 else None))
                         for key, value in statevector.get_bit_series().items()}
     # FIXME: figure out how to get access to low-latency frames outside
     # of the cluster. Until we figure that out, actual I/O errors have
     # to be non-fatal.
-    return {bitname.format(channel.split(':')[0], key):
+    return {bitname.format(ifo_from_channel(channel), key):
             None for key in bits if key is not None}
 
 
-@app.task(shared=False)
-def check_vectors(event, graceid, start, end):
+@app.task(shared=False, bind=True, default_retry_delay=5)
+def check_vectors(self, event, graceid, start, end):
     """Perform data quality checks for an event and labels/logs results to
     GraceDB.
 
@@ -410,7 +437,7 @@ def check_vectors(event, graceid, start, end):
 
     """
     # Skip early warning events (ie queries for times before now)
-    if start > Time.now().gps:
+    if end > Time.now().gps:
         log.info("Skipping detchar checks because %s is in the future",
                  event['graceid'])
         return event
@@ -429,7 +456,7 @@ def check_vectors(event, graceid, start, end):
     prepost_msg = "Check looked within -{}/+{} seconds of superevent. ".format(
         pre, post)
 
-    ifos = {key.split(':')[0] for key, val in
+    ifos = {ifo_from_channel(key) for key, val in
             app.conf['llhoft_channels'].items()}
     caches = {ifo: create_cache(ifo, start, end) for ifo in ifos}
     bit_defs = {channel_type: Bits(channel=bitdef['channel'],
@@ -445,31 +472,62 @@ def check_vectors(event, graceid, start, end):
         analysis_channels = {k: v for k, v in analysis_channels
                              if k[3:] != 'DMT-DQ_VECTOR'}.items()
     for channel, bits in analysis_channels:
-        states.update(check_vector(caches[channel.split(':')[0]], channel,
-                                   start, end, bit_defs[bits]))
+        try:
+            states.update(check_vector(
+                caches[ifo_from_channel(channel)], channel,
+                start, end, bit_defs[bits]))
+        except ValueError as exc:
+            # check_vector likely failed to find the requested data
+            # in the cache because it has yet to arrive
+            raise self.retry(exc=exc, max_retries=4)
     # Pick out DQ and injection states, then filter for active detectors
     dq_states = {key: value for key, value in states.items()
                  if key.split('_')[-1] != 'INJ'}
     inj_states = {key: value for key, value in states.items()
                   if key.split('_')[-1] == 'INJ'}
     active_dq_states = {key: value for key, value in dq_states.items()
-                        if key.split(':')[0] in instruments}
+                        if ifo_from_channel(key) in instruments}
     active_inj_states = {key: value for key, value in inj_states.items()
-                         if key.split(':')[0] in instruments}
+                         if ifo_from_channel(channel) in instruments}
 
-    # Check iDQ states
-    idq_probs = dict(check_idq(caches[channel.split(':')[0]],
-                               channel, start, end)
-                     for channel in app.conf['idq_channels'])
+    # Check iDQ states and filter for active instruments
+    idq_faps = dict(check_idq(caches[ifo_from_channel(channel)],
+                    channel, start, end)
+                    for channel in app.conf['idq_channels']
+                    if ifo_from_channel(channel) in instruments)
+    idq_oks = dict(check_idq(caches[ifo_from_channel(channel)],
+                   channel, start, end)
+                   for channel in app.conf['idq_ok_channels']
+                   if ifo_from_channel(channel) in instruments)
 
     # Logging iDQ to GraceDB
-    if None not in idq_probs.values():
-        idq_probs_readable = {k: round(v, 3) for k, v in idq_probs.items()}
-        if max(idq_probs.values()) >= app.conf['idq_pglitch_thresh']:
-            idq_msg = ("iDQ glitch probability is high: "
-                       "maximum p(glitch) is {}. ").format(
-                json.dumps(idq_probs_readable)[1:-1])
-            # If iDQ p(glitch) is high and pipeline enabled, apply DQV
+    # Checking the IDQ-OK vector
+    idq_not_ok_ifos = [
+        ifo_from_channel(ok_channel)
+        for ok_channel, min_value in idq_oks.items()
+        if min_value == 0 or min_value is None]
+    idq_not_ok_fap_chans = [
+        chan for chan in idq_faps.keys()
+        if ifo_from_channel(chan) in idq_not_ok_ifos]
+    # Remove iDQ FAP channels if their IDQ_OK values are bad
+    for idq_not_ok_chan in idq_not_ok_fap_chans:
+        del idq_faps[idq_not_ok_chan]
+    if len(idq_not_ok_ifos) > 0:
+        idq_ok_msg = (f"Not checking iDQ for {', '.join(idq_not_ok_ifos)} "
+                      "because it has times where IDQ_OK = 0. ")
+    else:
+        idq_ok_msg = ''
+
+    if None not in idq_faps.values() and len(idq_faps) > 0:
+        idq_faps_readable = {k: round(v, 3) for k, v in idq_faps.items()}
+        if min(idq_faps.values()) <= app.conf['idq_fap_thresh']:
+            idq_msg = ("iDQ false alarm probability is low "
+                       "(below {} threshold), "
+                       "i.e., there could be a data quality issue: "
+                       "minimum FAP is {}. ").format(
+                app.conf['idq_fap_thresh'],
+                json.dumps(idq_faps_readable)[1:-1])
+            # If iDQ FAP is low and pipeline enabled, apply DQV
             if app.conf['idq_veto'][pipeline]:
                 gracedb.remove_label('DQOK', graceid)
                 gracedb.create_label('DQV', graceid)
@@ -480,15 +538,18 @@ def check_vectors(event, graceid, start, end):
                 except ValueError:  # not in list
                     pass
         else:
-            idq_msg = ("iDQ glitch probabilities at both H1 and L1 "
-                       "are good (below {} threshold). "
-                       "Maximum p(glitch) is {}. ").format(
-                           app.conf['idq_pglitch_thresh'],
-                           json.dumps(idq_probs_readable)[1:-1])
+            idq_msg = ("iDQ false alarm probabilities for active detectors "
+                       "are good (above {} threshold). "
+                       "Minimum FAP is {}. ").format(
+                           app.conf['idq_fap_thresh'],
+                           json.dumps(idq_faps_readable)[1:-1])
+    elif None in idq_faps.values():
+        idq_msg = "iDQ false alarm probabilities unknown. "
     else:
-        idq_msg = "iDQ glitch probabilities unknown. "
+        idq_msg = ''
     gracedb.upload.delay(
-        None, None, graceid, idq_msg + prepost_msg, ['data_quality'])
+        None, None, graceid,
+        idq_ok_msg + idq_msg + prepost_msg, ['data_quality'])
 
     # Labeling INJ to GraceDB
     if False in active_inj_states.values():
@@ -564,7 +625,8 @@ def check_vectors(event, graceid, start, end):
         state = "unknown"
 
     # Create and upload DQR-compatible json
-    state_summary = '{} {} {}'.format(inj_msg, idq_msg, msg)
+    state_summary = '{} {} {}'.format(
+        inj_msg, idq_ok_msg + idq_msg, msg)
     if state == "unknown":
         json_state = "error"
     else:
