@@ -1,17 +1,14 @@
 """Qualitative source properties for CBC events."""
 import io
 import json
-from matplotlib import pyplot as plt
-
-from ligo.em_bright import em_bright
 
 from celery.utils.log import get_task_logger
+from matplotlib import pyplot as plt
 
-from ..import app
+from .. import app
+from ..util import NamedTemporaryFile, closing_figures
 from . import gracedb, igwn_alert
 from .p_astro import _format_prob
-from ..util import closing_figures, NamedTemporaryFile
-
 
 log = get_task_logger(__name__)
 
@@ -85,7 +82,7 @@ def plot(contents):
     return outfile.getvalue()
 
 
-@app.task(shared=False)
+@app.task(shared=False, queue='em-bright')
 def em_bright_posterior_samples(posterior_file_content):
     """Returns the probability of having a NS component and remnant
     using Bilby posterior samples.
@@ -107,10 +104,11 @@ def em_bright_posterior_samples(posterior_file_content):
     {"HasNS": 0.014904901243599122, "HasRemnant": 0.0, "HasMassGap": 0.0}
 
     """
+    from ligo.em_bright import em_bright
     with NamedTemporaryFile(content=posterior_file_content) as samplefile:
         filename = samplefile.name
         has_ns, has_remnant, has_massgap = em_bright.source_classification_pe(
-            filename, num_eos_draws=100, eos_seed=0
+            filename, num_eos_draws=10000, eos_seed=0
         )
     data = json.dumps({
         'HasNS': has_ns,
@@ -120,7 +118,7 @@ def em_bright_posterior_samples(posterior_file_content):
     return data
 
 
-@app.task(shared=False)
+@app.task(shared=False, queue='em-bright')
 def source_properties(mass1, mass2, spin1z, spin2z, snr):
     """Returns the probability of having a NS component, the probability of
     having non-zero disk mass, and the probability of any component being the
@@ -150,6 +148,7 @@ def source_properties(mass1, mass2, spin1z, spin2z, snr):
     >>> em_bright.source_properties(2.0, 1.0, 0.0, 0.0, 10.)
     '{"HasNS": 1.0, "HasRemnant": 1.0, "HasMassGap"}'
     """
+    from ligo.em_bright import em_bright
     p_ns, p_em, p_mg = em_bright.source_classification(
         mass1, mass2, spin1z, spin2z, snr
     )
