@@ -1,80 +1,37 @@
 """Source Parameter Estimation with LALInference, Bilby, and RapidPE."""
-from distutils.spawn import find_executable
-from distutils.dir_util import mkpath
 import glob
 import json
 import os
-import shutil
 import subprocess
 import urllib
+from distutils.dir_util import mkpath
+from distutils.spawn import find_executable
 
-from celery import group
-from gwdatafind import find_urls
 import numpy as np
+from bilby_pipe.bilbyargparser import BilbyConfigFileParser
+from bilby_pipe.utils import convert_string_to_dict
+from celery import group
+from celery.exceptions import Ignore
 
 from .. import app
 from ..jinja import env
-from . import condor
-from . import gracedb
+from . import condor, gracedb
 
+_RAPIDPE_NO_GSTLAL_TRIGGER_EXIT_CODE = 100
 
-def _data_exists(gpstime, frametype_dict):
-    """Check whether data at input GPS time can be found with gwdatafind and
-    return true if it is found.
-    """
-    return min(
-        len(
-            find_urls(ifo[0], frametype_dict[ifo], gpstime, gpstime + 1)
-        ) for ifo in frametype_dict.keys()
-    ) > 0
+RAPIDPE_GETENV = [
+    "DEFAULT_SEGMENT_SERVER", "GWDATAFIND_SERVER",
+    "LAL_DATA_PATH", "LD_LIBRARY_PATH", "LIBRARY_PATH",
+    "NDSSERVER", "PATH", "PYTHONPATH",
+]
+"""Names of environment variables to include in RapidPE HTCondor submit
+files."""
 
-
-class NotEnoughData(Exception):
-    """Raised if found data is not enough due to the latency of data
-    transfer
-    """
-
-
-@app.task(bind=True, autoretry_for=(NotEnoughData, ),
-          retry_backoff=True, retry_backoff_max=600, max_retries=16)
-def query_data(self, trigtime):
-    """Continues to query data until it is found with gwdatafind and return
-    frametypes for the data. This query will be retried for ~4600 seconds,
-    which is longer than the expected latency of transfer of high-latency data
-    with length of ~4000 seconds.
-    """
-    end = trigtime + 2
-    if _data_exists(end, app.conf['low_latency_frame_types']):
-        return app.conf['low_latency_frame_types']
-    elif _data_exists(end, app.conf['high_latency_frame_types']):
-        return app.conf['high_latency_frame_types']
-    else:
-        raise NotEnoughData
-
-
-@app.task(ignore_result=True, shared=False)
-def upload_no_frame_files(request, exc, traceback, superevent_id):
-    """Upload notification when no frame files are found.
-
-    Parameters
-    ----------
-    request : Context (placeholder)
-        Task request variables
-    exc : Exception
-        Exception raised by query_data
-    traceback : str (placeholder)
-        Traceback message from a task
-    superevent_id : str
-        The GraceDB ID of a target superevent
-
-    """
-    if isinstance(exc, NotEnoughData):
-        gracedb.upload.delay(
-            filecontents=None, filename=None,
-            graceid=superevent_id,
-            message='Frame files have not been found.',
-            tags='pe'
-        )
+RAPIDPE_ENVIRONMENT = {
+    "RIFT_LOWLATENCY": "True",
+}
+"""Names and values of environment variables to include in RapidPE HTCondor
+submit files."""
 
 
 def _find_appropriate_cal_env(trigtime, dir_name):
@@ -114,13 +71,11 @@ def _find_appropriate_cal_env(trigtime, dir_name):
     return os.path.join(dir_name, appropriate_cal.decode('utf-8'))
 
 
-def prepare_lalinference_ini(frametype_dict, event, superevent_id):
+def prepare_lalinference_ini(event, superevent_id):
     """Determine LALInference configurations and return ini file content
 
     Parameters
     ----------
-    frametype_dict : dict
-        Dictionary whose keys are ifos and values are frame types
     event : dict
         The json contents of a target G event retrieved from
         gracedb.get_event(), whose mass and spin information are used to
@@ -163,7 +118,7 @@ def prepare_lalinference_ini(frametype_dict, event, superevent_id):
                    'bayeswavepost': 'BayesWavePost'}
     ini_settings = {
         'gracedb_host': app.conf['gracedb_host'],
-        'types': frametype_dict,
+        'types': app.conf['low_latency_frame_types'],
         'channels': app.conf['strain_channel_names'],
         'state_vector_channels': app.conf['state_vector_channel_names'],
         'webdir': os.path.join(
@@ -192,8 +147,7 @@ def prepare_lalinference_ini(frametype_dict, event, superevent_id):
 
 
 @app.task(shared=False)
-def _setup_dag_for_lalinference(coinc, rundir, event, superevent_id,
-                                frametype_dict):
+def _setup_dag_for_lalinference(coinc, rundir, event, superevent_id):
     """Create DAG for a lalinference run and return the path to DAG.
 
     Parameters
@@ -208,8 +162,6 @@ def _setup_dag_for_lalinference(coinc, rundir, event, superevent_id,
         determine analysis settings.
     superevent_id : str
         The GraceDB ID of a target superevent
-    frametype_dict : dict
-        Dictionary whose keys are ifos and values are frame types
 
     Returns
     -------
@@ -223,8 +175,7 @@ def _setup_dag_for_lalinference(coinc, rundir, event, superevent_id,
         f.write(coinc)
 
     # write down and upload ini file
-    ini_contents = prepare_lalinference_ini(
-        frametype_dict, event, superevent_id)
+    ini_contents = prepare_lalinference_ini(event, superevent_id)
     path_to_ini = os.path.join(rundir, 'online_lalinference_pe.ini')
     with open(path_to_ini, 'w') as f:
         f.write(ini_contents)
@@ -248,14 +199,15 @@ def _setup_dag_for_lalinference(coinc, rundir, event, superevent_id,
             graceid=superevent_id,
             message='Failed to prepare DAG for lalinference', tags='pe'
         )
-        shutil.rmtree(rundir)
         raise
 
     return os.path.join(rundir, 'multidag.dag')
 
 
 @app.task(shared=False)
-def _setup_dag_for_bilby(coinc, rundir, event, superevent_id, mode):
+def _setup_dag_for_bilby(
+    coinc, rundir, event, superevent_id, mode="production"
+):
     """Create DAG for a bilby run and return the path to DAG.
 
     Parameters
@@ -271,13 +223,21 @@ def _setup_dag_for_bilby(coinc, rundir, event, superevent_id, mode):
     superevent_id : str
         The GraceDB ID of a target superevent
     mode : str
-        Analysis mode
+        Analysis mode, allowed options are "production" and "fast_test",
+        default is "production".
 
     Returns
     -------
     path_to_dag : str
         The path to the .dag file
 
+    Notes
+    -----
+    `--channel-dict o3replay` is added to bilby_pipe_gracedb arguments when the
+    gracedb host is different from `gracedb.ligo.org` or
+    `gracedb-test.ligo.org`. Condor queue is set to `Online_PE` if gracedb host
+    is `gracedb.ligo.org`, and `Online_PE_MDC` otherwise.
+
     """
     path_to_json = os.path.join(rundir, 'event.json')
     with open(path_to_json, 'w') as f:
@@ -288,7 +248,7 @@ def _setup_dag_for_bilby(coinc, rundir, event, superevent_id, mode):
         f.write(coinc)
 
     path_to_webdir = os.path.join(
-        app.conf['pe_results_path'], superevent_id, 'bilby'
+        app.conf['pe_results_path'], superevent_id, 'bilby', mode
     )
 
     path_to_settings = os.path.join(rundir, 'settings.json')
@@ -297,19 +257,61 @@ def _setup_dag_for_bilby(coinc, rundir, event, superevent_id, mode):
                  '--psd-file', path_to_psd, '--settings', path_to_settings]
     settings = {'summarypages_arguments': {'gracedb': event['graceid'],
                                            'no_ligo_skymap': True},
-                'queue': 'Online_PE'}
+                'accounting_user': 'soichiro.morisaki'}
+    if app.conf['gracedb_host'] != 'gracedb.ligo.org':
+        settings['queue'] = 'Online_PE_MDC'
+    else:
+        settings['queue'] = 'Online_PE'
+        settings['accounting'] = 'ligo.prod.o4.cbc.pe.bilby'
+    # FIXME: using live data for gracedb-test events should be reconsidered
+    # when we have a better idea to differentiate MDC and real events.
+    if app.conf['gracedb_host'] not in [
+        'gracedb.ligo.org', 'gracedb-test.ligo.org'
+    ]:
+        setup_arg += ['--channel-dict', 'o3replay']
 
-    if mode == 'fast_bns':
-        # TODO: add cbc likelihood mode here once bilby_pipe is released
-        settings.update(
-            {'sampler_kwargs': {'nact': 3, 'nlive': 500, 'npool': 24},
-             'n_parallel': 2,
-             'request_cpus': 24,
-             'request_memory_generation': 8.0}
+    trigger_chirp_mass = event['extra_attributes']['CoincInspiral']['mchirp']
+    if trigger_chirp_mass < 0.6:
+        raise ValueError(
+            "No bilby settings available for trigger chirp mass of"
+            f" {trigger_chirp_mass}Msun."
         )
+    if mode == 'production':
+        settings.update(
+            {
+                'sampler_kwargs': {'naccept': 60, 'nlive': 500,
+                                   'npool': 24, 'sample': 'acceptance-walk'},
+                'n_parallel': 2,
+                'request_cpus': 24,
+                'spline_calibration_nodes': 10,
+                'request_memory_generation': 8.0
+            }
+        )
+        # use low-spin IMRPhenomD below chirp mass of m1=3Msun, m2=1Msun
+        # assuming binary neutron star
+        if trigger_chirp_mass < 1.465:
+            likelihood_mode = 'lowspin_phenomd_fhigh1024_roq'
+            settings['sampler_kwargs']['naccept'] = 10
+        # use IMRPhenomPv2 with mass ratio upper bound of 8 below chirp mass of
+        # m1=8Msun, m2=1Msun
+        elif trigger_chirp_mass < 2.243:
+            likelihood_mode = 'phenompv2_bns_roq'
+        # use IMRPhenomPv2 with mass ratio upper bound of 20 in chirp-mass
+        # range where IMRPhenomXPHM ROQ bases are not available
+        elif trigger_chirp_mass < 12:
+            likelihood_mode = 'low_q_phenompv2_roq'
+        else:
+            likelihood_mode = 'phenomxphm_roq'
+            settings['request_memory_generation'] = 36.0
+            settings['request_memory'] = 16.0
+        setup_arg += ['--cbc-likelihood-mode', likelihood_mode]
     elif mode == 'fast_test':
-        setup_arg += ['--channel-dict', 'o3replay',
-                      '--sampler-kwargs', 'FastTest']
+        setup_arg += ["--sampler-kwargs", "FastTest"]
+        if trigger_chirp_mass < 3.9:
+            setup_arg += ['--cbc-likelihood-mode', 'phenompv2_bns_roq']
+            settings['request_memory_generation'] = 8.0
+    else:
+        raise ValueError(f"mode: {mode} not recognized.")
 
     with open(path_to_settings, 'w') as f:
         json.dump(settings, f, indent=2)
@@ -322,19 +324,21 @@ def _setup_dag_for_bilby(coinc, rundir, event, superevent_id, mode):
         gracedb.upload.delay(
             filecontents=contents, filename='bilby_dag.log',
             graceid=superevent_id,
-            message=f'Failed to prepare DAG for {mode} bilby', tags='pe'
+            message=f'Failed to prepare DAG for {mode}-mode bilby', tags='pe'
         )
-        shutil.rmtree(rundir)
         raise
     else:
         # Uploads bilby ini file to GraceDB
         with open(os.path.join(rundir, 'bilby_config.ini'), 'r') as f:
             ini_contents = f.read()
+        if mode == 'production':
+            filename = 'bilby_config.ini'
+        else:
+            filename = f'bilby_{mode}_config.ini'
         gracedb.upload.delay(
-            ini_contents, filename=f'bilby_{mode}_config.ini',
-            graceid=superevent_id,
-            message=(f'Automatically generated {mode} Bilby configuration file'
-                     ' for this event.'),
+            ini_contents, filename=filename, graceid=superevent_id,
+            message=(f'Automatically generated {mode}-mode Bilby configuration'
+                     ' file for this event.'),
             tags='pe')
 
     path_to_dag, = glob.glob(os.path.join(rundir, 'submit/dag*.submit'))
@@ -342,7 +346,7 @@ def _setup_dag_for_bilby(coinc, rundir, event, superevent_id, mode):
 
 
 @app.task(shared=False)
-def _setup_dag_for_rapidpe(rundir, superevent_id, frametype_dict):
+def _setup_dag_for_rapidpe(rundir, superevent_id, event):
     """Create DAG for a rapidpe run and return the path to DAG.
 
     Parameters
@@ -351,8 +355,6 @@ def _setup_dag_for_rapidpe(rundir, superevent_id, frametype_dict):
         The path to a run directory where the DAG file is created
     superevent_id : str
         The GraceDB ID of a target superevent
-    frametype_dict : dict
-        Dictionary whose keys are ifos and values are frame types
 
     Returns
     -------
@@ -360,13 +362,11 @@ def _setup_dag_for_rapidpe(rundir, superevent_id, frametype_dict):
         The path to the .dag file
 
     """
-    # dump SVD depth file
-    path_to_svd_file = os.path.join(rundir, 'svd_depth_methods.json')
-    with open(path_to_svd_file, 'w') as f:
-        json.dump(
-            [{'bounds': {}, 'fudge_factors': {'mchirp': 0.2, 'eta': 0.1},
-             'svd_depth': 1}],
-            f)
+    gracedb_host = app.conf['gracedb_host']
+
+    settings = app.conf['rapidpe_settings']
+    trigger_snr = event['extra_attributes']['CoincInspiral']['snr']
+    high_snr_trigger = trigger_snr >= 37.5
 
     # dump ini file
     ini_template = env.get_template('rapidpe.jinja2')
@@ -375,10 +375,16 @@ def _setup_dag_for_rapidpe(rundir, superevent_id, frametype_dict):
          'webdir': os.path.join(
              app.conf['pe_results_path'], superevent_id, 'rapidpe'
          ),
-         'gracedb_url': f'https://{app.conf["gracedb_host"]}/api',
+         'gracedb_url': f'https://{gracedb_host}/api',
          'superevent_id': superevent_id,
-         'frame_data_types': frametype_dict,
-         'svd_depth_json': os.path.abspath(path_to_svd_file)})
+         'run_mode': settings['run_mode'],
+         'frame_data_types': app.conf['low_latency_frame_types'],
+         'accounting_group': settings['accounting_group'],
+         'use_cprofile': settings['use_cprofile'],
+         'gracedb_host': gracedb_host,
+         'high_snr_trigger': high_snr_trigger,
+         'getenv': RAPIDPE_GETENV,
+         'environment': RAPIDPE_ENVIRONMENT})
     path_to_ini = os.path.join(rundir, 'rapidpe.ini')
     with open(path_to_ini, 'w') as f:
         f.write(ini_contents)
@@ -396,31 +402,46 @@ def _setup_dag_for_rapidpe(rundir, superevent_id, frametype_dict):
     except subprocess.CalledProcessError as e:
         contents = b'args:\n' + json.dumps(e.args[1]).encode('utf-8') + \
                    b'\n\nstdout:\n' + e.stdout + b'\n\nstderr:\n' + e.stderr
+
+        message = 'Failed to prepare DAG for Rapid PE'
+
+        fail_gracefully = False
+        if e.returncode == _RAPIDPE_NO_GSTLAL_TRIGGER_EXIT_CODE:
+            fail_gracefully = True
+            message += ": no GstLAL trigger available"
+
         gracedb.upload.delay(
             filecontents=contents, filename='rapidpe_dag.log',
             graceid=superevent_id,
-            message='Failed to prepare DAG for Rapid PE', tags='pe'
+            message=message, tags='pe'
         )
-        shutil.rmtree(rundir)
-        raise
+
+        if fail_gracefully:
+            # Ends task but without logging as a failure
+            raise Ignore()
+        else:
+            # Ends task with the unhandled error logged
+            raise
 
     # return path to dag
     dag = os.path.join(rundir, "event_all_iterations.dag")
-    os.chdir(os.path.dirname(dag))
-    dag = os.path.basename(dag)
     return dag
 
 
 @app.task(shared=False)
-def _condor_no_submit(path_to_dag):
+def _condor_no_submit(path_to_dag, include_env=None):
     """Run 'condor_submit_dag -no_submit' and return the path to .sub file."""
-    subprocess.run(['condor_submit_dag', '-no_submit', path_to_dag],
-                   capture_output=True, check=True)
+    args = ['condor_submit_dag']
+
+    if include_env is not None:
+        args += ['-include_env', ','.join(include_env)]
+
+    args += ['-no_submit', path_to_dag]
+    subprocess.run(args, capture_output=True, check=True)
     return '{}.condor.sub'.format(path_to_dag)
 
 
-def dag_prepare_task(rundir, event, superevent_id, pe_pipeline,
-                     frametype_dict=None, **kwargs):
+def dag_prepare_task(rundir, event, superevent_id, pe_pipeline, **kwargs):
     """Return a canvas of tasks to prepare DAG.
 
     Parameters
@@ -436,8 +457,6 @@ def dag_prepare_task(rundir, event, superevent_id, pe_pipeline,
     pe_pipeline : str
         The parameter estimation pipeline used,
         lalinference, bilby, or rapidpe.
-    frametype_dict : dict
-        Dictionary whose keys are ifos and values are frame types
 
     Returns
     -------
@@ -445,20 +464,24 @@ def dag_prepare_task(rundir, event, superevent_id, pe_pipeline,
         The canvas of tasks to prepare DAG
 
     """
+    # List of environment variables `condor_submit_dag` should be aware of.
+    include_env = None
+
     if pe_pipeline == 'lalinference':
         canvas = gracedb.download.si('coinc.xml', event['graceid']) | \
-            _setup_dag_for_lalinference.s(rundir, event, superevent_id,
-                                          frametype_dict)
+            _setup_dag_for_lalinference.s(rundir, event, superevent_id)
     elif pe_pipeline == 'bilby':
         canvas = gracedb.download.si('coinc.xml', event['graceid']) | \
             _setup_dag_for_bilby.s(
                 rundir, event, superevent_id, kwargs['bilby_mode'])
     elif pe_pipeline == 'rapidpe':
-        canvas = _setup_dag_for_rapidpe.s(
-            rundir, superevent_id, frametype_dict)
+        canvas = _setup_dag_for_rapidpe.s(rundir, superevent_id, event)
+        include_env = RAPIDPE_GETENV
     else:
         raise NotImplementedError(f'Unknown PE pipeline {pe_pipeline}.')
-    canvas |= _condor_no_submit.s()
+
+    canvas |= _condor_no_submit.s(include_env=include_env)
+
     return canvas
 
 
@@ -483,6 +506,25 @@ def _find_paths_from_name(directory, name):
     return glob.iglob(os.path.join(directory, '**', name), recursive=True)
 
 
+@app.task(ignore_result=True, shared=False)
+def _clean_up_bilby(rundir):
+    """Remove large data products produced by bilby
+
+    Parameters
+    ----------
+    rundir : str
+
+    """
+    for p in glob.glob(
+        os.path.join(rundir, "data/*_generation_roq_weights.hdf5")
+    ):
+        os.remove(p)
+    for p in glob.glob(
+        os.path.join(rundir, "data/*_generation_data_dump.pickle")
+    ):
+        os.remove(p)
+
+
 @app.task(ignore_result=True, shared=False)
 def job_error_notification(request, exc, traceback,
                            superevent_id, rundir, analysis):
@@ -503,6 +545,11 @@ def job_error_notification(request, exc, traceback,
     analysis : str
         Analysis name used as a label in uploaded messages
 
+    Notes
+    -----
+    Some large bilby data products are cleaned up after the notification if the
+    gracedb host is different from `gracedb.ligo.org`.
+
     """
     if isinstance(exc, condor.JobRunning):
         subprocess.run(['condor_rm', str(exc.args[0]['Cluster'])])
@@ -522,8 +569,14 @@ def job_error_notification(request, exc, traceback,
             message=f'The {analysis} condor job failed.'
         )
 
-    # upload all the .log, .err, and .out files
-    for filename in ['*.log', '*.err', '*.out']:
+    if analysis == "rapidpe":
+        to_upload = [
+            'event_all_iterations.dag.lib.err',
+            'marginalize_extrinsic_parameters_iteration_*.dag.lib.err'
+        ]
+    else:
+        to_upload = ['*.log', '*.err', '*.out']
+    for filename in to_upload:
         tasks = []
         for path in _find_paths_from_name(rundir, filename):
             with open(path, 'rb') as f:
@@ -541,22 +594,12 @@ def job_error_notification(request, exc, traceback,
                 ))
         canvas |= group(tasks)
 
+    if "bilby" in analysis and app.conf['gracedb_host'] != 'gracedb.ligo.org':
+        canvas |= _clean_up_bilby.si(rundir)
+
     canvas.delay()
 
 
-@app.task(ignore_result=True, shared=False)
-def clean_up(rundir):
-    """Clean up a run directory.
-
-    Parameters
-    ----------
-    rundir : str
-        The path to a run directory where the DAG file exits
-
-    """
-    shutil.rmtree(rundir)
-
-
 def _upload_tasks_lalinference(rundir, superevent_id):
     """Return canvas of tasks to upload LALInference results
 
@@ -656,10 +699,18 @@ def _upload_tasks_bilby(rundir, superevent_id, mode):
     tasks : canvas
         The work-flow for uploading Bilby results
 
+    Notes
+    -----
+    Some large bilby data products are cleaned up after posteterior file is
+    uploaded if the gracedb host is different from `gracedb.ligo.org`.
+
     """
     # convert bilby sample file into one compatible with ligo-skymap
     samples_dir = os.path.join(rundir, 'final_result')
-    samples_filename = f'Bilby.{mode}.posterior_samples.hdf5'
+    if mode == 'production':
+        samples_filename = 'Bilby.posterior_samples.hdf5'
+    else:
+        samples_filename = f'Bilby.{mode}.posterior_samples.hdf5'
     out_samples = os.path.join(samples_dir, samples_filename)
     in_samples, = glob.glob(os.path.join(samples_dir, '*result.hdf5'))
     subprocess.run(
@@ -669,24 +720,80 @@ def _upload_tasks_bilby(rundir, superevent_id, mode):
     with open(out_samples, 'rb') as f:
         canvas = gracedb.upload.si(
             f.read(), samples_filename,
-            superevent_id, f'{mode} Bilby posterior samples', 'pe')
+            superevent_id, f'{mode}-mode Bilby posterior samples', 'pe')
 
-    # plots
+    if app.conf['gracedb_host'] != 'gracedb.ligo.org':
+        canvas |= _clean_up_bilby.si(rundir)
+
+    # pesummary
+    pesummary_kwargs = {}
+    path_to_ini, = glob.glob(os.path.join(rundir, "*_complete.ini"))
+    pesummary_kwargs["config"] = path_to_ini
+    config_parser = BilbyConfigFileParser()
+    with open(path_to_ini, "r") as f:
+        config_content, _, _, _ = config_parser.parse(f)
+    pesummary_kwargs["psd"] = convert_string_to_dict(
+        config_content["psd-dict"]
+    )
+    pesummary_kwargs["calibration"] = convert_string_to_dict(
+        config_content["spline-calibration-envelope-dict"]
+    )
+    pesummary_kwargs["approximant"] = config_content["waveform-approximant"]
+    pesummary_kwargs["f_low"] = config_content["minimum-frequency"]
+    pesummary_kwargs["f_ref"] = config_content["reference-frequency"]
+    pesummary_kwargs["label"] = "online"
+
+    webdir = os.path.join(config_content["webdir"], 'pesummary')
+    url = urllib.parse.urljoin(
+        app.conf['pe_results_url'],
+        os.path.relpath(
+            os.path.join(webdir, 'home.html'),
+            app.conf['pe_results_path']
+        )
+    )
+    canvas = group(
+        canvas,
+        _pesummary_task(webdir, in_samples, **pesummary_kwargs)
+        |
+        gracedb.upload.si(
+            None, None, superevent_id,
+            f'PESummary page for {mode}-mode Bilby is available '
+            f'<a href={url}>here</a>',
+            'pe'
+        )
+    )
+
+    return canvas
+
+
+def _upload_tasks_rapidpe(rundir, superevent_id):
+    summary_path = os.path.join(rundir, "summary")
+
+    url = urllib.parse.urljoin(
+        app.conf['pe_results_url'],
+        os.path.join(superevent_id, 'rapidpe', 'summarypage.html')
+    )
+    canvas = gracedb.upload.si(
+        None, None, superevent_id,
+        f'Summary page for RapidPE-RIFT is available <a href={url}>here</a>',
+        ('pe',))
+    to_upload = [
+        (
+            "p_astro.json", "RapidPE_RIFT.p_astro.json",
+            "RapidPE-RIFT Pastro results",
+            ("pe", "p_astro", "public"),
+        ),
+    ]
     tasks = []
-    resultdir = os.path.join(rundir, 'result')
-    for parameter_type in ['extrinsic', 'intrinsic']:
-        # Here it is not required that only a single png file exists, so that
-        # posterior samples are uploaded whatever.
-        for path in glob.iglob(
-            os.path.join(resultdir, f'*_{parameter_type}_corner.png')
-        ):
-            with open(path, 'rb') as f:
-                tasks.append(gracedb.upload.si(
-                    f.read(), f'Bilby.{mode}.{parameter_type}.png',
-                    superevent_id,
-                    f'{mode} Bilby corner plot for {parameter_type} '
-                    'parameters', 'pe'
-                ))
+    for src_basename, dst_filename, description, tags in to_upload:
+        src_filename = os.path.join(summary_path, src_basename)
+        if os.path.isfile(src_filename):
+            with open(src_filename, "rb") as f:
+                tasks.append(
+                    gracedb.upload.si(
+                        f.read(), dst_filename,
+                        superevent_id, description, tags))
+
     canvas |= group(tasks)
 
     return canvas
@@ -694,7 +801,7 @@ def _upload_tasks_bilby(rundir, superevent_id, mode):
 
 @app.task(ignore_result=True, shared=False)
 def dag_finished(rundir, superevent_id, pe_pipeline, **kwargs):
-    """Upload PE results and clean up run directory
+    """Upload PE results
 
     Parameters
     ----------
@@ -707,37 +814,82 @@ def dag_finished(rundir, superevent_id, pe_pipeline, **kwargs):
         lalinference, bilby, or rapidpe.
 
     """
-    if pe_pipeline == 'rapidpe':
-        os.chdir(os.path.expanduser("~"))
-
     if pe_pipeline == 'lalinference':
         canvas = _upload_tasks_lalinference(rundir, superevent_id)
     elif pe_pipeline == 'bilby':
         canvas = _upload_tasks_bilby(
             rundir, superevent_id, kwargs['bilby_mode'])
     elif pe_pipeline == 'rapidpe':
-        # TODO: upload rapidpe posterior samples
-        canvas = gracedb.upload.si(
-            None, None, superevent_id,
-            'Online RapidPE-RIFT parameter estimation finished.', 'pe')
+        canvas = _upload_tasks_rapidpe(rundir, superevent_id)
     else:
         raise NotImplementedError(f'Unknown PE pipeline {pe_pipeline}.')
 
-    canvas = canvas | clean_up.si(rundir)
     canvas.delay()
 
+    # NOTE: check if this should include rapidpe as well
     if pe_pipeline == 'bilby':
         gracedb.create_label.delay('PE_READY', superevent_id)
 
 
+def _pesummary_task(webdir, samples, **pesummary_kwargs):
+    """Return a celery task to submit a pesummary condor job.
+
+    Parameters
+    ----------
+    webdir : str
+        output directory
+    samples : str
+        path to posterior sample file
+    **pesummary_kwargs
+        Extra arguments of summarypages
+
+    Returns
+    -------
+    celery task
+
+    Notes
+    -----
+    `--disable_interactive --disable_expert` are added and `--redshift_method
+    exact --evolve_spins_forwards` are not added to `summarypages` arguments
+    when the gracedb host is different from `gracedb.ligo.org`. Condor queue is
+    set to `Online_PE` if gracedb host is `gracedb.ligo.org`, and
+    `Online_PE_MDC` otherwise.
+
+    """
+    args = [
+        "summarypages", "--webdir", webdir, "--samples", samples, "--gw",
+        "--no_ligo_skymap", "--multi_process", "6"
+    ]
+    for key in pesummary_kwargs:
+        if key in ["psd", "calibration"]:
+            args += [f"--{key}"]
+            for ifo in pesummary_kwargs[key]:
+                args += [f'{ifo}:{pesummary_kwargs[key][ifo]}']
+        else:
+            args += [f"--{key}", pesummary_kwargs[key]]
+    condor_kwargs = dict(
+        request_memory=16000, request_disk=5000, request_cpus=6,
+        accounting_group_user='soichiro.morisaki'
+    )
+    if app.conf['gracedb_host'] != 'gracedb.ligo.org':
+        condor_kwargs['accounting_group'] = 'ligo.dev.o4.cbc.pe.bilby'
+        condor_kwargs['requirements'] = '((TARGET.Online_PE_MDC =?= True))'
+        condor_kwargs['+Online_PE_MDC'] = True
+        args += ["--disable_interactive", "--disable_expert"]
+    else:
+        condor_kwargs['accounting_group'] = 'ligo.prod.o4.cbc.pe.bilby'
+        condor_kwargs['requirements'] = '((TARGET.Online_PE =?= True))'
+        condor_kwargs['+Online_PE'] = True
+        args += ["--redshift_method", "exact", "--evolve_spins_forwards"]
+    return condor.check_output.si(args, **condor_kwargs)
+
+
 @app.task(ignore_result=True, shared=False)
-def start_pe(frametype_dict, event, superevent_id, pe_pipeline):
+def start_pe(event, superevent_id, pe_pipeline):
     """Run Parameter Estimation on a given event.
 
     Parameters
     ----------
-    frametype_dict : dict
-        Dictionary whose keys are ifos and values are frame types
     event : dict
         The json contents of a target G event retrieved from
         gracedb.get_event(), whose mass and spin information are used to
@@ -753,21 +905,57 @@ def start_pe(frametype_dict, event, superevent_id, pe_pipeline):
     pipeline_dir = os.path.expanduser('~/.cache/{}'.format(pe_pipeline))
     mkpath(pipeline_dir)
     event_dir = os.path.join(pipeline_dir, superevent_id)
-    os.mkdir(event_dir)
 
     if pe_pipeline == 'bilby':
-        modes = [app.conf['bilby_default_mode']]
-        # add quick-bns mode if chirp mass is lower than that of 3Msun-3Msun
-        if event['extra_attributes']['CoincInspiral']['mchirp'] < 2.62:
-            modes += ['quick-bns']
+        if (
+            app.conf['gracedb_host'] == 'gracedb-playground.ligo.org' and
+            event['extra_attributes']['CoincInspiral']['mchirp'] >= 12
+        ):
+            # Count the number of BBH jobs and do not start a run if it exceeds
+            # 5 so that we do not use up disk space. We assume that the job is
+            # running if a data dump pickle file exists under the run
+            # directory, which is the largest file produced by PE and removed
+            # when the run completes.
+            number_of_bbh_running = 0
+            for p in glob.glob(
+                os.path.join(
+                    pipeline_dir,
+                    "*/*/data/*_generation_data_dump.pickle"
+                )
+            ):
+                path_to_ev = os.path.join(os.path.dirname(p), "../event.json")
+                if os.path.exists(path_to_ev):
+                    with open(path_to_ev, "r") as f:
+                        ev = json.load(f)
+                        mc = ev['extra_attributes']['CoincInspiral']['mchirp']
+                    if mc >= 12:
+                        number_of_bbh_running += 1
+            if number_of_bbh_running > 5:
+                gracedb.upload.delay(
+                    filecontents=None, filename=None, graceid=superevent_id,
+                    message='Parameter estimation will not start to save disk '
+                            f'space (There are {number_of_bbh_running} BBH '
+                            'jobs running).',
+                    tags='pe'
+                )
+                return
+        modes = ["production"]
         rundirs = [os.path.join(event_dir, m) for m in modes]
         kwargs_list = [{'bilby_mode': m} for m in modes]
-        analyses = [f'{m} bilby' for m in modes]
+        analyses = [f'{m}-mode bilby' for m in modes]
+        condor_submit_task = condor.submit
+    elif pe_pipeline == 'rapidpe':
+        rundirs = [event_dir]
+        kwargs_list = [{'event_pipeline': event["pipeline"]}]
+        analyses = [pe_pipeline]
+        condor_submit_task = condor.submit_rapidpe
     else:
         rundirs = [event_dir]
         kwargs_list = [{}]
         analyses = [pe_pipeline]
+        condor_submit_task = condor.submit
 
+    os.mkdir(event_dir)
     for rundir, kwargs, analysis in zip(rundirs, kwargs_list, analyses):
         mkpath(rundir)
 
@@ -780,11 +968,10 @@ def start_pe(frametype_dict, event, superevent_id, pe_pipeline):
 
         (
             dag_prepare_task(
-                rundir, event, superevent_id, pe_pipeline, frametype_dict,
-                **kwargs
+                rundir, event, superevent_id, pe_pipeline, **kwargs
             )
             |
-            condor.submit.s().on_error(
+            condor_submit_task.s().on_error(
                 job_error_notification.s(superevent_id, rundir, analysis)
             )
             |
