@@ -6,13 +6,18 @@ import io
 import json
 
 from celery.utils.log import get_task_logger
-from ligo import p_astro_computation as pastrocomp
-from matplotlib import pyplot as plt
-import numpy as np
 
-from . import gracedb, igwn_alert
+try:
+    from ligo.p_astro import computation as pastrocomp
+except ImportError:  # p_astro older than lscsoft/p_astro!42
+    from ligo import p_astro_computation as pastrocomp
+
+import numpy as np
+from matplotlib import pyplot as plt
+
 from .. import app
-from ..util import closing_figures, PromiseProxy, read_json
+from ..util import PromiseProxy, closing_figures, read_json
+from . import gracedb, igwn_alert
 
 MEAN_VALUES_DICT = PromiseProxy(
     read_json, ('ligo.data', 'H1L1V1-mean_counts-1126051217-61603201.json'))
@@ -168,12 +173,16 @@ def handle(alert):
     """LVAlert handler to plot and upload a visualization of every
     ``*.p_astro.json`` file that is added to a superevent.
     """
+    if alert['alert_type'] != 'log':
+        return
+
     graceid = alert['uid']
     filename = alert['data'].get('filename')
     p_astro_filenames = {f'{pipeline}.p_astro.json' for pipeline in
-                         ['gstlal', 'mbta', 'pycbc', 'spiir']}
+                         ['cwb', 'gstlal', 'mbta', 'pycbc', 'spiir',
+                          'RapidPE_RIFT']}
 
-    if alert['alert_type'] == 'log' and filename in p_astro_filenames:
+    if filename in p_astro_filenames:
         (
             gracedb.download.s(filename, graceid)
             |
