@@ -1,27 +1,28 @@
 """Annotations for sky maps."""
-import io
 import os
 import tempfile
 
-from astropy.io import fits
 from astropy import table
+from astropy.io import fits
 from celery import group
 from celery.exceptions import Ignore
-from ligo.skymap.tool import ligo_skymap_flatten
-from ligo.skymap.tool import ligo_skymap_unflatten
-from ligo.skymap.tool import ligo_skymap_from_samples
-from ligo.skymap.tool import ligo_skymap_plot
-from ligo.skymap.tool import ligo_skymap_plot_volume
+from ligo.skymap.tool import (ligo_skymap_flatten, ligo_skymap_from_samples,
+                              ligo_skymap_plot, ligo_skymap_plot_coherence,
+                              ligo_skymap_plot_volume, ligo_skymap_unflatten)
 from matplotlib import pyplot as plt
-import numpy as np
 
-from . import gracedb
-from . import igwn_alert
-from ..import app
+from .. import app
 from ..jinja import env
 from ..util.cmdline import handling_system_exit
 from ..util.matplotlib import closing_figures
 from ..util.tempfile import NamedTemporaryFile
+from . import external_skymaps, gracedb, igwn_alert
+
+
+@app.task(ignore_result=True, shared=False)
+def annotate_fits_tuple(filecontents_versioned_filename, graceid, tags):
+    filecontents, versioned_filename = filecontents_versioned_filename
+    annotate_fits(filecontents, versioned_filename, graceid, tags)
 
 
 @app.task(ignore_result=True, shared=False)
@@ -56,7 +57,7 @@ def annotate_fits(filecontents, versioned_filename, graceid, tags):
         '{versioned_filename}">{versioned_filename}</a>').format(
             graceid=graceid, versioned_filename=versioned_filename)
     flatten_msg = (
-        'Flat-resolution fits file created from '
+        'Flat-resolution FITS file created from '
         '<a href="/api/superevents/{graceid}/files/'
         '{versioned_filename}">{versioned_filename}</a>').format(
             graceid=graceid, versioned_filename=versioned_filename)
@@ -178,108 +179,20 @@ def unflatten(filecontents, filename):
         return open(outfilename, 'rb').read()
 
 
-@app.task(shared=False, queue='openmp')
-def skymap_from_samples(samplefilecontents):
-    """Generate multi-resolution fits file from samples."""
+@app.task(shared=False, queue='multiprocessing')
+def skymap_from_samples(samplefilecontents, superevent_id, instruments):
+    """Generate multi-resolution FITS file from samples."""
     with NamedTemporaryFile(content=samplefilecontents) as samplefile, \
             tempfile.TemporaryDirectory() as tmpdir, \
             handling_system_exit():
-        ligo_skymap_from_samples.main(
-            ['-j', '-o', tmpdir, samplefile.name])
+        ligo_skymap_from_samples.main([
+            '-j', '--seed', '150914', '--maxpts', '5000', '--objid',
+            superevent_id, '--instruments', *instruments, '-o', tmpdir,
+            samplefile.name])
         with open(os.path.join(tmpdir, 'skymap.fits'), 'rb') as f:
             return f.read()
 
 
-def plot_bayes_factor(logb,
-                      values=(1, 3, 5),
-                      labels=('', 'strong', 'very strong'),
-                      xlim=7, title=None, palette='RdYlBu',
-                      var_label="B"):
-    """Visualize a Bayes factor as a `bullet graph`_.
-
-    Make a bar chart of a log Bayes factor as compared to a set of subjective
-    threshold values. By default, use the thresholds from
-    Kass & Raftery (1995).
-
-    .. _`bullet graph`: https://en.wikipedia.org/wiki/Bullet_graph
-
-    Parameters
-    ----------
-    logb : float
-        The natural logarithm of the Bayes factor.
-    values : list
-        A list of floating point values for human-friendly confidence levels.
-    labels : list
-        A list of string labels for human-friendly confidence levels.
-    xlim : float
-        Limits of plot (`-xlim` to `+xlim`).
-    title : str
-        Title for plot.
-    palette : str
-        Color palette.
-    var_label : str
-        The variable symbol used in plotting
-
-    Returns
-    -------
-    fig : Matplotlib figure
-    ax : Matplotlib axes
-
-    Example
-    -------
-    .. plot::
-        :alt: Example Bayes factor plot
-
-        from gwcelery.tasks.skymaps import plot_bayes_factor
-        plot_bayes_factor(6.3, title='GWCelery is awesome')
-
-    """
-    with plt.style.context('seaborn-notebook'):
-        fig, ax = plt.subplots(figsize=(6, 1.7), tight_layout=True)
-        ax.set_xlim(-xlim, xlim)
-        ax.set_ylim(-0.5, 0.5)
-        ax.set_yticks([])
-        ax.set_title(title)
-        ax.set_ylabel(r'$\ln\,{}$'.format(var_label), rotation=0,
-                      rotation_mode='anchor',
-                      ha='right', va='center')
-
-        # Add human-friendly labels
-        ticks = (*(-x for x in reversed(values)), 0, *values)
-        ticklabels = (
-            *(f'{s}\nevidence\nagainst'.strip() for s in reversed(labels)), '',
-            *(f'{s}\nevidence\nfor'.strip() for s in labels))
-        ax.set_xticks(ticks)
-        ax.set_xticklabels(ticklabels)
-        plt.setp(ax.get_xticklines(), visible=False)
-        plt.setp(ax.get_xticklabels()[:len(ticks) // 2], ha='right')
-        plt.setp(ax.get_xticklabels()[len(ticks) // 2:], ha='left')
-
-        # Plot colored bands for confidence thresholds
-        fmt = plt.FuncFormatter(lambda x, _: f'{x:+g}'.replace('+0', '0'))
-        ax2 = ax.twiny()
-        ax2.set_xlim(*ax.get_xlim())
-        ax2.set_xticks(ticks)
-        ax2.xaxis.set_major_formatter(fmt)
-        levels = (-xlim, *ticks, xlim)
-        colors = plt.get_cmap(palette)(np.arange(1, len(levels)) / len(levels))
-        ax.barh(0, np.diff(levels), 1, levels[:-1],
-                linewidth=plt.rcParams['xtick.major.width'],
-                color=colors, edgecolor='white')
-
-        # Plot bar for log Bayes factor value
-        ax.barh(0, logb, 0.5, color='black',
-                linewidth=plt.rcParams['xtick.major.width'],
-                edgecolor='white')
-
-        for ax_ in fig.axes:
-            ax_.grid(False)
-            for spine in ax_.spines.values():
-                spine.set_visible(False)
-
-    return fig, ax
-
-
 @app.task(shared=False)
 @closing_figures()
 def plot_coherence(filecontents):
@@ -303,20 +216,15 @@ def plot_coherence(filecontents):
     # Explicitly use a non-interactive Matplotlib backend.
     plt.switch_backend('agg')
 
-    with NamedTemporaryFile(content=filecontents) as fitsfile:
+    with NamedTemporaryFile(mode='rb', suffix='.png') as pngfile, \
+            NamedTemporaryFile(content=filecontents) as fitsfile:
         header = fits.getheader(fitsfile, 1)
-    try:
-        logb = header['LOGBCI']
-    except KeyError:
-        raise Ignore('FITS file does not have a LOGBCI field')
-
-    objid = header['OBJECT']
-    logb_string = np.format_float_positional(logb, 1, trim='0', sign=True)
-    fig, _ = plot_bayes_factor(
-        logb, title=rf'Coherence of {objid} $[\ln\,B = {logb_string}]$')
-    outfile = io.BytesIO()
-    fig.savefig(outfile, format='png')
-    return outfile.getvalue()
+        try:
+            header['LOGBCI']
+        except KeyError:
+            raise Ignore('FITS file does not have a LOGBCI field')
+        ligo_skymap_plot_coherence.main([fitsfile.name, '-o', pngfile.name])
+        return pngfile.read()
 
 
 @igwn_alert.handler('superevent',
@@ -333,7 +241,9 @@ def handle_plot_coherence(alert):
     """
     if alert['alert_type'] != 'log':
         return  # not for us
-    if not alert['data']['filename'].endswith('.fits'):
+    if not alert['data']['filename'].endswith('.fits') or \
+            (alert['data']['filename'] ==
+             external_skymaps.COMBINED_SKYMAP_FILENAME_MULTIORDER):
         return  # not for us
 
     graceid = alert['uid']
