@@ -6,12 +6,12 @@ superevents.
 *   Primary logic to respond to low latency triggers contained in
     :meth:`process` function.
 """
-from itertools import filterfalse
+from itertools import filterfalse, groupby
 
 from celery.utils.log import get_task_logger
 from ligo.segments import segment, segmentlist
 
-from ..import app
+from .. import app
 from . import gracedb, igwn_alert
 
 log = get_task_logger(__name__)
@@ -24,18 +24,34 @@ REQUIRED_LABELS_BY_GROUP = {
 be complete.
 """
 
-FROZEN_LABEL = 'EM_Selected'
+FROZEN_LABEL = 'LOW_SIGNIF_LOCKED'
 """This label indicates that the superevent manager should make no further
 changes to the preferred event."""
 
+SIGNIFICANT_LABEL = 'SIGNIF_LOCKED'
+"""This label indicates that the superevent is elevated to significant"""
+
 EARLY_WARNING_LABEL = 'EARLY_WARNING'
 """This label indicates that the superevent contains a significant
 early warning event."""
 
+EARLY_WARNING_SEARCH_NAME = 'EarlyWarning'
+"""Search name for Early Warning searches. Only significant events
+result in consideration by the superevent manager."""
+
+SUBSOLAR_SEARCH_NAME = 'SSM'
+"""Search name for subsolar mass search. These are ignored by
+the superevent manager."""
+
 READY_LABEL = 'EM_READY'
 """This label indicates that a preferred event has been assigned and it
 has all data products required to make it ready for annotations."""
 
+VT_SEARCH_NAME = 'VTInjection'
+"""Search name for events uploaded as a part of estimating the spacetime
+sensitive volume. Events from this search do not form superevents, and
+are not annotated."""
+
 
 @igwn_alert.handler('cbc_gstlal',
                     'cbc_spiir',
@@ -43,6 +59,7 @@ has all data products required to make it ready for annotations."""
                     'cbc_mbta',
                     'burst_olib',
                     'burst_cwb',
+                    'burst_mly',
                     shared=False)
 def handle(payload):
     """Respond to IGWN alert topics from low-latency search pipelines and
@@ -50,6 +67,19 @@ def handle(payload):
     """
     alert_type = payload['alert_type']
     gid = payload['object']['graceid']
+    alert_group = payload['object']['group'].lower()
+    alert_search = payload['object']['search']
+
+    ifos = get_instruments(payload['object']) if alert_group == 'cbc' \
+        else payload['object']['instruments'].split(',')
+    # Ignore inclusion of events involving KAGRA; revert when policy is changed
+    if "K1" in ifos:
+        log.info('Skipping %s because it involves KAGRA data', gid)
+        return
+    # Ignore inclusion of events from VT search
+    if alert_search == VT_SEARCH_NAME:
+        log.info("Skipping {} event {}".format(VT_SEARCH_NAME, gid))
+        return
 
     try:
         far = payload['object']['far']
@@ -66,6 +96,11 @@ def handle(payload):
         priority = 0
         label = payload['data']['name']
         group = payload['object']['group'].lower()
+        search = payload['object']['search'].lower()
+        pipeline = payload['object']['pipeline'].lower()
+        # Special case of cWB-BBH that require the same label of CBC
+        if search == 'bbh' and pipeline == 'cwb':
+            group = 'cbc'
         if label == 'RAVEN_ALERT':
             log.info('Label %s added to %s', label, gid)
         elif label not in REQUIRED_LABELS_BY_GROUP[group]:
@@ -76,11 +111,10 @@ def handle(payload):
             return
     elif alert_type != 'new':
         return
-
     process.si(payload).apply_async(priority=priority)
 
 
-@gracedb.task(queue='superevent', shared=False)
+@gracedb.task(queue='superevent', shared=False, time_limit=600)
 @gracedb.catch_retryable_http_errors
 def process(payload):
     """
@@ -174,17 +208,17 @@ def process(payload):
                 sid = gracedb.create_superevent(event_info['graceid'],
                                                 t_0, t_start, t_end)
 
-    if should_publish(event_info):
+    if should_publish(event_info, significant=True):
         gracedb.create_label.delay('ADVREQ', sid)
         if is_complete(event_info):
-            if app.conf['preliminary_alert_timeout'] \
-                    and EARLY_WARNING_LABEL not in event_info['labels']:
-                gracedb.create_label.s(FROZEN_LABEL, sid).set(
-                    queue='superevent',
-                    countdown=app.conf['preliminary_alert_timeout']
-                ).delay()
-            else:  # fast path if no countdown
-                gracedb.create_label(FROZEN_LABEL, sid)
+            if EARLY_WARNING_LABEL in event_info['labels']:
+                gracedb.create_label(EARLY_WARNING_LABEL, sid)
+            else:
+                gracedb.create_label(SIGNIFICANT_LABEL, sid)
+
+    if should_publish(event_info, significant=False):
+        if is_complete(event_info):
+            gracedb.create_label(FROZEN_LABEL, sid)
 
 
 def get_category(event):
@@ -212,7 +246,7 @@ def get_category(event):
 def get_ts(event):
     """Get time extent of an event, depending on pipeline-specific parameters.
 
-    *   For CWB, use the event's ``duration`` field.
+    *   For CWB and MLy, use the event's ``duration`` field.
     *   For oLIB, use the ratio of the event's ``quality_mean`` and
         ``frequency_mean`` fields.
     *   For all other pipelines, use the
@@ -245,6 +279,9 @@ def get_ts(event):
         attribs = event['extra_attributes']['LalInferenceBurst']
         d_t_start = d_t_end = (attribs['quality_mean'] /
                                attribs['frequency_mean'])
+    elif pipeline == 'mly':
+        attribs = event['extra_attributes']['MLyBurst']
+        d_t_start = d_t_end = attribs['duration']
     else:
         d_t_start = app.conf['superevent_d_t_start'].get(
             pipeline, app.conf['superevent_default_d_t_start'])
@@ -283,6 +320,9 @@ def get_snr(event):
     elif pipeline == 'olib':
         attribs = event['extra_attributes']['LalInferenceBurst']
         return attribs['omicron_snr_network']
+    elif pipeline == 'mly':
+        attribs = event['extra_attributes']['MLyBurst']
+        return attribs['SNR']
     else:
         raise NotImplementedError('SNR attribute not found')
 
@@ -349,6 +389,32 @@ def get_instruments_in_ranking_statistic(event):
                 if single.get('chisq') is not None}
 
 
+@app.task(shared=False)
+def select_pipeline_preferred_event(events):
+    """Group list of G events by pipeline, and apply :meth:`keyfunc`
+    to select the pipeline preferred events.
+
+    Parameters
+    ----------
+    events : list
+        list of event dictionaries
+
+    Returns
+    -------
+    dict
+        pipeline, graceid pairs
+    """
+    g_events = list(
+        filterfalse(lambda x: x['group'] == "External", events))
+    g_events_by_pipeline = groupby(
+        sorted(g_events, key=lambda x: x['pipeline']),
+        key=lambda x: x['pipeline']
+    )
+
+    return dict(
+        (k, select_preferred_event(g)) for k, g in g_events_by_pipeline)
+
+
 @app.task(shared=False)
 def select_preferred_event(events):
     """Select the preferred event out of a list of G events, typically
@@ -360,9 +426,8 @@ def select_preferred_event(events):
         list of event dictionaries
 
     """
-    # FIXME: Requires robust determination of an External event
     g_events = list(
-        filterfalse(lambda x: x['graceid'].startswith('E'), events))
+        filterfalse(lambda x: x['group'] == "External", events))
     return max(g_events, key=keyfunc)
 
 
@@ -383,12 +448,18 @@ def is_complete(event):
 
     """
     group = event['group'].lower()
+    pipeline = event['pipeline'].lower()
+    search = event['search'].lower()
     label_set = set(event['labels'])
+    # Define the special case burst-cwb-bbh that is a CBC
+    if pipeline == 'cwb' and search == 'bbh':
+        group = 'cbc'
+
     required_labels = REQUIRED_LABELS_BY_GROUP[group]
     return required_labels.issubset(label_set)
 
 
-def should_publish(event):
+def should_publish(event, significant=True):
     """Determine whether an event should be published as a public alert.
 
     All of the following conditions must be true for a public alert:
@@ -396,9 +467,11 @@ def should_publish(event):
     *   The event's ``offline`` flag is not set.
     *   The event's false alarm rate, weighted by the group-specific trials
         factor as specified by the
-        :obj:`~gwcelery.conf.preliminary_alert_trials_factor` configuration
-        setting, is less than or equal to
-        :obj:`~gwcelery.conf.preliminary_alert_far_threshold`.
+        :obj:`~gwcelery.conf.preliminary_alert_trials_factor`
+        (:obj:`~gwcelery.conf.significant_alert_trials_factor`)
+        configuration setting, is less than or equal to
+        :obj:`~gwcelery.conf.preliminary_alert_far_threshold`
+        (:obj:`~gwcelery.conf.significant_alert_far_threshold`)
 
     Parameters
     ----------
@@ -406,6 +479,12 @@ def should_publish(event):
         Event dictionary (e.g., the return value from
         :meth:`gwcelery.tasks.gracedb.get_event`, or
         ``preferred_event_data`` in igwn-alert packet.)
+    significant : bool
+        Flag to use significant
+        (:obj:`~gwcelery.conf.significant_alert_far_threshold`),
+        or less-significant
+        (:obj:`~gwcelery.conf.preliminary_alert_far_threshold`)
+        FAR threshold.
 
     Returns
     -------
@@ -414,20 +493,28 @@ def should_publish(event):
         :obj:`False` if it does not.
 
     """
-    return all(_should_publish(event))
+    return all(_should_publish(event, significant=significant))
 
 
-def _should_publish(event):
+def _should_publish(event, significant=False):
     """Wrapper around :meth:`should_publish`. Returns the boolean returns of
     the publishability criteria as a tuple for later use.
     """
     group = event['group'].lower()
+    search = event.get('search', '').lower()
+    pipeline = event.get('pipeline', '').lower()
+    # Define the special case burst-cwb-bbh that is a CBC
+    if pipeline == 'cwb' and search == 'bbh':
+        group = 'cbc'
     if EARLY_WARNING_LABEL in event['labels']:
         far_threshold = app.conf['early_warning_alert_far_threshold']
         trials_factor = app.conf['early_warning_alert_trials_factor']
-    else:
+    elif not significant:
         far_threshold = app.conf['preliminary_alert_far_threshold'][group]
         trials_factor = app.conf['preliminary_alert_trials_factor'][group]
+    else:
+        far_threshold = app.conf['significant_alert_far_threshold'][group]
+        trials_factor = app.conf['significant_alert_trials_factor'][group]
     far = trials_factor * event['far']
     raven_coincidence = ('RAVEN_ALERT' in event['labels'])
 
@@ -460,24 +547,39 @@ def keyfunc(event):
 
     """
     group = event['group'].lower()
+    search = event['search'].lower()
     try:
-        group_rank = ['burst', 'cbc'].index(group)
-    except ValueError:
+        group_rank = app.conf['superevent_candidate_preference'][group].get(
+            search, -1
+        )
+    except KeyError:
         group_rank = -1
-
     if group == 'cbc':
-        group_rank = 1
         n_ifos = len(get_instruments(event))
-        significance = get_snr(event)
+        snr_or_far_ranking = get_snr(event)
     else:
         # We don't care about the number of detectors for burst events.
         n_ifos = -1
         # Smaller FAR -> higher IFAR -> more significant.
         # Use -FAR instead of IFAR=1/FAR so that rank for FAR=0 is defined.
-        significance = -event['far']
+        snr_or_far_ranking = -event['far']
 
-    return (is_complete(event), *_should_publish(event), group_rank, n_ifos,
-            significance)
+    # Conditions that determine choice of the preferred event
+    # event completeness comes first
+    # then, publishability criteria for significant events
+    # then, publishability criteria for less-significant events
+    # then, CBC group is given higher priority over Burst
+    # then, prioritize more number of detectors
+    # finally, use SNR (FAR) between two CBC (Burst) events
+    # See https://rtd.igwn.org/projects/gwcelery/en/latest/gwcelery.tasks.superevents.html#selection-of-the-preferred-event  # noqa: E501
+    return (
+        is_complete(event),
+        *_should_publish(event, significant=True),
+        *_should_publish(event, significant=False),
+        group_rank,
+        n_ifos,
+        snr_or_far_ranking
+    )
 
 
 def _update_superevent(superevent_id, new_event_dict,
@@ -514,7 +616,7 @@ def _update_superevent(superevent_id, new_event_dict,
         kwargs['t_end'] = t_end
     if FROZEN_LABEL not in superevent_labels:
         if keyfunc(new_event_dict) > keyfunc(preferred_event_dict):
-            # update preferred event when EM_Selected is not applied
+            # update preferred event when LOW_SIGNIF_LOCKED is not applied
             kwargs['t_0'] = t_0
             kwargs['preferred_event'] = new_event_dict['graceid']
 
