@@ -1,5 +1,6 @@
 import subprocess
 
+import celery.exceptions
 import pytest
 
 from ..tasks import condor
@@ -82,6 +83,7 @@ def mock_condor_submit(monkeypatch):
     monkeypatch.setattr('subprocess.run', mock_run)
 
 
+@pytest.mark.live_worker
 def test_check_output_error_on_submit(monkeypatch):
     """Test capturing an error from condor_submit."""
     accounting_group = 'foo.bar'
@@ -93,35 +95,39 @@ def test_check_output_error_on_submit(monkeypatch):
 
     monkeypatch.setattr('subprocess.run', mock_run)
 
+    result = condor.check_output.delay(cmd, accounting_group=accounting_group)
     with pytest.raises(subprocess.CalledProcessError) as exc_info:
-        condor.check_output.delay(cmd, accounting_group=accounting_group)
+        result.get()
     assert 'accounting_group=' + accounting_group in exc_info.value.cmd
     assert exc_info.value.output == msg
 
 
+@pytest.mark.live_worker
 def test_check_output_aborted(mock_condor_submit_aborted):
     """Test a job that is aborted."""
+    result = condor.check_output.delay(['sleep', '1'])
     with pytest.raises(condor.JobAborted):
-        condor.check_output.delay(['sleep', '1'])
+        result.get()
 
 
+@pytest.mark.live_worker
 def test_check_output_fails(mock_condor_submit):
     """Test a job that immediately fails."""
+    result = condor.check_output.delay(['sleep', '--foo="bar bat"', '1'])
     with pytest.raises(condor.JobFailed) as exc_info:
-        condor.check_output.delay(['sleep', '--foo="bar bat"', '1'])
+        result.get()
     assert exc_info.value.returncode == 1
 
 
-# FIXME: this test doesn't work in eager mode.
-# def test_check_output_running(mock_condor_submit_running):
-#
-#     with pytest.raises(condor.JobRunning):
-#         condor.check_output.delay(['sleep', '1'])
+@pytest.mark.live_worker
+def test_check_output_running(mock_condor_submit_running):
+    result = condor.check_output.delay(['sleep', '1'])
+    with pytest.raises(celery.exceptions.TimeoutError):
+        result.get(2)
 
 
-# FIXME: this test doesn't work in eager mode.
-# See https://github.com/celery/celery/issues/4661.
-# def test_check_output_succeeds(mock_condor_submit):
-#     """Test a job that immediately succeeds."""
-#
-#     condor.check_output.delay(['sleep', '1'])
+@pytest.mark.live_worker
+def test_check_output_succeeds(mock_condor_submit):
+    """Test a job that immediately succeeds."""
+    result = condor.check_output.delay(['sleep', '1'])
+    result.get()
