@@ -2,9 +2,9 @@ from unittest.mock import call, patch
 
 import pytest
 
+from .. import app
+from ..tasks import gracedb, raven
 from .test_tasks_skymaps import toy_fits_filecontents  # noqa: F401
-from ..tasks import gracedb
-from ..tasks import raven
 
 
 @pytest.mark.live_worker
@@ -16,7 +16,10 @@ from ..tasks import raven
      ['Burst', 'T4', ['SNEWS'], 'Supernova', [], -10, 10],
      ['CBC', 'MS4', ['Fermi'], ['MDC'], [], -1, 5],
      ['CBC', 'E1', ['Fermi'], [], ['GRB'], -5, 1],
-     ['CBC', 'M1', ['Fermi'], [], ['MDC'], -5, 1]])
+     ['CBC', 'M1', ['Fermi'], [], ['MDC'], -5, 1],
+     ['CBC', 'E1', ['Fermi'], ['SubGRB'], [], -11, 1],
+     ['Burst', 'E1', ['Swift'], ['SubGRBTargeted'], [], -20, 10],
+     [None, 'E1', ['Swift'], ['SubGRBTargeted'], [], -20, 10]])
 @patch('gwcelery.tasks.gracedb.create_label')
 @patch('gwcelery.tasks.raven.raven_pipeline.s')
 @patch('gwcelery.tasks.raven.search.si', return_value=[{'superevent_id': 'S5',
@@ -41,6 +44,13 @@ def test_coincidence_search(mock_calculate_coincidence_far,
         gracedb_id, alert_object, tl, th, group)
 
 
+@pytest.mark.parametrize(
+    'group,search', [['CBC', 'SubGRBTargeted'], ['Test', 'GRB']])
+def test_raven_window_errors(group, search):
+    with pytest.raises(ValueError):
+        raven._time_window('S1', group, ['INTEGRAL'], [search])
+
+
 @pytest.mark.parametrize(
     'event_type,event_id', [['SE', 'S1234'], ['ExtTrig', 'E1234']])
 @patch('ligo.raven.search.search')
@@ -77,7 +87,8 @@ def test_calculate_coincidence_far(
         'S1234', 'E4321', tl, th,
         se_dict=se, ext_dict=ext,
         incl_sky=False, grb_search='GRB',
-        gracedb=gracedb.client, far_grb=None)
+        gracedb=gracedb.client, far_grb=None,
+        far_gw_thresh=None, far_grb_thresh=None)
 
 
 @patch('ligo.raven.search.calc_signif_gracedb')
@@ -85,16 +96,20 @@ def test_calculate_coincidence_far_subgrb(mock_calc_signif):
     se = {'superevent_id': 'S1234'}
     ext = {'graceid': 'E4321',
            'pipeline': 'Fermi',
-           'search': 'GRB',
+           'search': 'SubGRBTargeted',
            'labels': [],
-           'far': 1e5}
+           'far': 1e-5}
     tl, th = -1, 10
     raven.calculate_coincidence_far(se, ext, tl, th)
     mock_calc_signif.assert_called_once_with(
         'S1234', 'E4321', tl, th,
         se_dict=se, ext_dict=ext,
-        incl_sky=False, grb_search='GRB',
-        gracedb=gracedb.client, far_grb=1e5)
+        incl_sky=False, grb_search='SubGRBTargeted',
+        gracedb=gracedb.client, far_grb=ext['far'],
+        far_gw_thresh=(
+            app.conf['raven_targeted_far_thresholds']['GW']['Fermi']),
+        far_grb_thresh=(
+            app.conf['raven_targeted_far_thresholds']['GRB']['Fermi']))
 
 
 @pytest.mark.parametrize('group', ['CBC', 'Burst'])  # noqa: F811
@@ -121,13 +136,14 @@ def test_calculate_spacetime_coincidence_far_fermi(
         se_fitsfile='fermi_skymap.fits.gz',
         ext_fitsfile='fermi_skymap.fits.gz',
         se_moc=True, ext_moc=False,
-        use_radec=False,
-        gracedb=gracedb.client, far_grb=None)
+        gracedb=gracedb.client, far_grb=None,
+        far_gw_thresh=None, far_grb_thresh=None,
+        use_preferred_event_skymap=False)
 
 
 @pytest.mark.parametrize('group', ['CBC', 'Burst'])  # noqa: F811
 @patch('gwcelery.tasks.external_skymaps.get_skymap_filename',
-       return_value='swift_skymap.fits.gz')
+       return_value='swift_skymap.multiorder.fits')
 @patch('ligo.raven.search.calc_signif_gracedb')
 def test_calculate_spacetime_coincidence_far_swift(
         mock_calc_signif, mock_get_skymap_filename, group):
@@ -146,11 +162,57 @@ def test_calculate_spacetime_coincidence_far_swift(
         'S1234', 'E4321', tl, th,
         incl_sky=True, grb_search='GRB',
         se_dict=se, ext_dict=ext,
-        se_fitsfile='swift_skymap.fits.gz',
-        ext_fitsfile='swift_skymap.fits.gz',
+        se_fitsfile='swift_skymap.multiorder.fits',
+        ext_fitsfile='swift_skymap.multiorder.fits',
+        se_moc=True, ext_moc=True,
+        gracedb=gracedb.client, far_grb=None,
+        far_gw_thresh=None, far_grb_thresh=None,
+        use_preferred_event_skymap=False)
+
+
+@pytest.mark.parametrize('group', ['CBC', 'Burst'])  # noqa: F811
+@patch('gwcelery.tasks.external_skymaps.get_skymap_filename',
+       return_value='fermi_skymap.fits.gz')
+@patch('ligo.raven.search.calc_signif_gracedb')
+def test_calculate_spacetime_coincidence_far_preferred(
+        mock_calc_signif, mock_get_skymap_filename, group):
+    se = {'superevent_id': 'S1234',
+          'preferred_event': 'G1'}
+    ext = {'graceid': 'E4321',
+           'pipeline': 'Fermi',
+           'search': 'GRB',
+           'labels': ['EXT_SKYMAP_READY', 'EM_READY'],
+           'far': None}
+    if group == 'CBC':
+        tl, th = -5, 1
+    else:
+        tl, th = -600, 60
+    raven.calculate_coincidence_far(se, ext, tl, th)
+    mock_calc_signif.assert_called_once_with(
+        'S1234', 'E4321', tl, th,
+        incl_sky=True, grb_search='GRB',
+        se_dict=se, ext_dict=ext,
+        se_fitsfile='fermi_skymap.fits.gz',
+        ext_fitsfile='fermi_skymap.fits.gz',
         se_moc=True, ext_moc=False,
-        use_radec=True,
-        gracedb=gracedb.client, far_grb=None)
+        gracedb=gracedb.client, far_grb=None,
+        far_gw_thresh=None, far_grb_thresh=None,
+        use_preferred_event_skymap=True)
+
+
+@patch('ligo.raven.search.calc_signif_gracedb')
+def test_calculate_coincidence_far_snews(
+        mock_calc_signif):
+    se = {'superevent_id': 'S1234'}
+    ext = {'graceid': 'E4321',
+           'pipeline': 'SNEWS',
+           'search': 'Supernova',
+           'labels': [],
+           'far': None}
+    tl, th = -600, 60
+    raven.calculate_coincidence_far(se, ext, tl, th)
+    # We should not calculate the joint FAR for SNEWS candidates
+    mock_calc_signif.assert_not_called()
 
 
 def mock_get_labels(superevent_id):
@@ -187,7 +249,9 @@ def mock_get_superevent(superevent_id):
         old_space_far = None
     return {'time_coinc_far': old_time_far,
             'space_coinc_far': old_space_far,
-            'superevent_id': superevent_id}
+            'superevent_id': superevent_id,
+            'labels': [],
+            'em_type': None}
 
 
 @pytest.mark.parametrize(
@@ -205,18 +269,16 @@ def mock_get_superevent(superevent_id):
      [[{'superevent_id': 'S12', 'far': .001, 'preferred_event': 'G3'}],
         'T4', -10, 10, 'Burst'],
      [[{'superevent_id': 'MS13', 'far': 1, 'preferred_event': 'M3'}],
-        'M15', -1, 5, 'CBC']])
+        'M15', -1, 5, 'CBC'],
+     [[{'superevent_id': 'S13', 'far': 1, 'preferred_event': 'G4'}],
+        'E6', -60, 600, 'Burst']])
 @patch('gwcelery.tasks.raven.trigger_raven_alert.run')
 @patch('gwcelery.tasks.gracedb.get_labels', mock_get_labels)
 @patch('gwcelery.tasks.raven.calculate_coincidence_far.run',
        return_value=mock_coinc_far())
-@patch('gwcelery.tasks.gracedb.update_superevent')
 @patch('gwcelery.tasks.gracedb.create_label.run')
-@patch('gwcelery.tasks.external_skymaps.plot_overlap_integral.run')
 @patch('gwcelery.tasks.gracedb.get_superevent', mock_get_superevent)
-def test_raven_pipeline(mock_plot_overlap_integral,
-                        mock_create_label,
-                        mock_update_superevent,
+def test_raven_pipeline(mock_create_label,
                         mock_calculate_coincidence_far,
                         mock_trigger_raven_alert,
                         raven_search_results, graceid, tl, th, group):
@@ -227,9 +289,8 @@ def test_raven_pipeline(mock_plot_overlap_integral,
     """
     alert_object = {'preferred_event': 'G1', 'pipeline': 'Fermi',
                     'search': 'Supernova' if graceid == 'T4' else 'GRB',
-                    'labels': []}
-    time_coinc_far = mock_coinc_far()['temporal_coinc_far']
-    space_coinc_far = mock_coinc_far()['spatiotemporal_coinc_far']
+                    'labels': [],
+                    'superevent': None if graceid != 'E6' else 'S14'}
 
     for result in raven_search_results:
         result['labels'] = []
@@ -247,13 +308,13 @@ def test_raven_pipeline(mock_plot_overlap_integral,
         alert_object['time_coinc_far'] = 1e-5
         alert_object['space_coinc_far'] = None
         alert_object['overlap_integral'] = None
+        for result in raven_search_results:
+            result['superevent'] = None
     raven.raven_pipeline(raven_search_results, graceid, alert_object, tl, th,
                          group)
 
     coinc_calls = []
     label_calls = []
-    update_calls = []
-    plot_calls = []
     if not raven_search_results:
         mock_calculate_coincidence_far.assert_not_called()
         mock_create_label.assert_not_called()
@@ -261,26 +322,15 @@ def test_raven_pipeline(mock_plot_overlap_integral,
     if 'S' in graceid:
         for result in raven_search_results:
             label_calls.append(call('EM_COINC', result['graceid']))
-            coinc_calls.append(call(alert_object, result, tl, th))
+            coinc_calls.append(call(alert_object, result, tl, th,
+                                    use_superevent_skymap=None))
             label_calls.append(call('EM_COINC', graceid))
-            update_calls.append(
-                call(graceid, em_type=result['graceid'],
-                     time_coinc_far=time_coinc_far,
-                     space_coinc_far=space_coinc_far))
-            plot_calls.append(
-                call(mock_coinc_far(), graceid, result['graceid']))
     else:
         result = raven.preferred_superevent(raven_search_results)[0]
         label_calls.append(call('EM_COINC', result['superevent_id']))
-        coinc_calls.append(call(result, alert_object, tl, th))
+        coinc_calls.append(call(result, alert_object, tl, th,
+                                use_superevent_skymap=None))
         label_calls.append(call('EM_COINC', graceid))
-        update_calls.append(
-            call(raven_search_results[-1]['superevent_id'],
-                 em_type=graceid,
-                 time_coinc_far=time_coinc_far,
-                 space_coinc_far=space_coinc_far))
-        plot_calls.append(
-            call(mock_coinc_far(), result['superevent_id'], graceid))
 
     alert_calls = []
     if 'S' in graceid:
@@ -291,13 +341,16 @@ def test_raven_pipeline(mock_plot_overlap_integral,
         alert_calls.append(call(
             mock_coinc_far(),
             result, graceid, alert_object, group))
-    mock_trigger_raven_alert.assert_has_calls(alert_calls, any_order=True)
+    if graceid != 'E6':
+        mock_trigger_raven_alert.assert_has_calls(alert_calls, any_order=True)
 
-    mock_calculate_coincidence_far.assert_has_calls(coinc_calls,
-                                                    any_order=True)
-    mock_update_superevent.assert_has_calls(update_calls, any_order=True)
-    mock_create_label.assert_has_calls(label_calls, any_order=True)
-    mock_plot_overlap_integral.assert_has_calls(plot_calls, any_order=True)
+        mock_calculate_coincidence_far.assert_has_calls(coinc_calls,
+                                                        any_order=True)
+        mock_create_label.assert_has_calls(label_calls, any_order=True)
+    else:
+        mock_trigger_raven_alert.assert_not_called()
+        mock_calculate_coincidence_far.assert_not_called()
+        mock_create_label.assert_not_called()
 
 
 @pytest.mark.parametrize(
@@ -338,8 +391,11 @@ def test_update_superevent(mock_update_superevent,
                            new_time_far, new_space_far,
                            superevent_id,
                            pipeline, result):
-
-    superevent = {'superevent_id': superevent_id}
+    """This function tests that new coincidences update the superevent when
+    they have a superior joint FAR or are a SNEWS event.
+    """
+    superevent = {'superevent_id': superevent_id,
+                  'labels': []}
     ext_event = {'graceid': 'E100',
                  'pipeline': pipeline}
     coinc_far_dict = {'temporal_coinc_far': new_time_far,
@@ -354,6 +410,76 @@ def test_update_superevent(mock_update_superevent,
         mock_update_superevent.assert_not_called()
 
 
+def mock_get_event(graceid):
+    if 'E' in graceid:
+        if graceid == "E1":
+            pipeline, search, labels = 'Fermi', 'GRB', []
+        elif graceid == "E2":
+            pipeline, search, labels = 'Fermi', 'GRB', ['NOT_GRB']
+        elif graceid == "E3":
+            pipeline, search, labels = 'SNEWS', 'Supernova', []
+        elif graceid == "E4":
+            pipeline, search, labels = 'Fermi', 'GRB', []
+        elif graceid == "E5":
+            pipeline, search, labels = 'Fermi', 'GRB', ['RAVEN_ALERT']
+        return {'superevent_id': graceid.replace('E', 'S'),
+                'graceid': graceid,
+                'pipeline': pipeline,
+                'search': search,
+                'labels': labels}
+    elif 'S' in graceid:
+        return {'em_type': graceid.replace('S', 'E'),
+                'time_coinc_far': 1e-20,
+                'space_coinc_far': 1e-20,
+                'superevent_id': graceid,
+                'labels': ['COMBINEDSKYMAP_READY'] if graceid == 'S4' else []}
+
+
+@pytest.mark.parametrize(
+    'superevent_id,ext_event_id_old,ext_event_id_new,result',
+    # NOT_GRB should not supersede real GRB
+    [['S1', 'E1', 'E2', False],
+     # Real GRB should supersede NOT_GRB
+     ['S2', 'E2', 'E1', True],
+     # SNEWS event should not be overwritten by GRB
+     ['S3', 'E3', 'E1', False],
+     # RAVEN_ALERT should supersede subthreshold joint candidate
+     ['S4', 'E4', 'E5', True],
+     # Subthreshold joint candidate should not overwrite RAVEN_ALERT
+     ['S5', 'E5', 'E4', False]]
+)
+@patch('gwcelery.tasks.gracedb.update_superevent')
+@patch('gwcelery.tasks.gracedb.get_event', mock_get_event)
+@patch('gwcelery.tasks.gracedb.get_superevent', mock_get_event)
+def test_update_superevent_w_emtype(mock_update_superevent,
+                                    superevent_id,
+                                    ext_event_id_old, ext_event_id_new,
+                                    result):
+    """This function tests that new coincidences update the superevent
+    if the new GRB is likely a real GRB and the old is likely not.
+    Also tested is that SNEWS event aren't overwritten by GRBs and locked
+    events are overwritten, denote by the COMBINEDSKYMAP_READY label in the
+    superevent which is applied just before sending an alert.
+    """
+    superevent = mock_get_event(superevent_id)
+    superevent['em_type'] = ext_event_id_old
+    ext_event_new = mock_get_event(ext_event_id_new)
+    ext_event_old = mock_get_event(ext_event_id_old)
+    time_coinc_far = 1e-10 if ext_event_old['labels'] else 1e-20
+    space_coinc_far = 1e-10 if ext_event_old['labels'] else 1e-30
+    coinc_far_dict = {'temporal_coinc_far': time_coinc_far,
+                      'spatiotemporal_coinc_far': space_coinc_far}
+    raven.update_coinc_far(coinc_far_dict, superevent, ext_event_new)
+    if result:
+        mock_update_superevent.assert_called_with(
+            superevent_id,
+            em_type=ext_event_id_new if result else ext_event_id_old,
+            time_coinc_far=time_coinc_far,
+            space_coinc_far=space_coinc_far)
+    else:
+        mock_update_superevent.assert_not_called()
+
+
 def _mock_get_event(graceid):
     if graceid == "S1234":
         return {"superevent_id": "S1234",
@@ -385,6 +511,11 @@ def _mock_get_event(graceid):
                 "preferred_event": "G000003",
                 "preferred_event_data": {"group": "Burst"},
                 "far": 1e-6}
+    elif graceid == "S9999":
+        return {"superevent_id": "S9876",
+                "preferred_event": "G000003",
+                "preferred_event_data": {"group": "Burst"},
+                "far": -1e-6}
     elif graceid == "TS1111":
         return {"superevent_id": "TS1111",
                 "preferred_event": "G000003",
@@ -425,12 +556,30 @@ def _mock_get_event(graceid):
                 "search": 'GRB',
                 "labels": ['NOT_GRB'],
                 "group": 'External'}
-    elif graceid == 'M5':
-        return {"graceid": "M5",
+    elif graceid == 'M6':
+        return {"graceid": "M6",
                 "pipeline": 'Fermi',
                 "search": 'MDC',
                 "labels": [],
                 "group": 'External'}
+    elif graceid == 'E7':
+        return {"graceid": "E7",
+                "pipeline": 'Fermi',
+                "search": 'GRB',
+                "labels": [],
+                "group": 'External'}
+    elif graceid == 'E8':
+        return {"graceid": "E8",
+                "pipeline": 'Fermi',
+                "search": 'SubGRBTargeted',
+                "labels": [],
+                "group": 'External'}
+    elif graceid == 'E9':
+        return {"graceid": "E9",
+                "pipeline": 'Fermi',
+                "search": 'SubGRBTargeted',
+                "labels": ['NOT_GRB'],
+                "group": 'External'}
     else:
         raise AssertionError
 
@@ -458,10 +607,17 @@ def _mock_get_coinc_far(graceid):
     elif graceid == "MS1111":
         return {"temporal_coinc_far": 1e-09,
                 "spatiotemporal_coinc_far": 1e-10}
+    elif graceid == "S9999":
+        return {"temporal_coinc_far": -1e-09,
+                "spatiotemporal_coinc_far": -1e-10}
     else:
         return {}
 
 
+def _return_coinc_far_dict(coinc_far_dict, *args):
+    return coinc_far_dict
+
+
 @pytest.mark.parametrize(
     'graceid,result_id,group,expected_result',
     [['S1234', 'E1', 'Burst', False],
@@ -469,18 +625,30 @@ def _mock_get_coinc_far(graceid):
      ['S2468', 'E5', 'CBC', False],
      ['S2468', 'E2', 'CBC', False],
      ['S2345', 'E3', 'Burst', True],
-     ['MS1111', 'M5', 'CBC', True],
+     ['MS1111', 'M6', 'CBC', True],
+     ['S5678', 'E1', 'CBC', False],
+     ['S5678', 'E7', 'CBC', False],
      ['E1', 'S9876', 'CBC', True],
      ['E1', 'S2468', 'CBC', False],
      ['E2', 'S5678', 'CBC', False],
      ['E3', 'S8642', 'Burst', False],
      ['E3', 'S9876', 'Burst', True],
      ['T4', 'TS1111', 'CBC', False],
-     ['E5', 'S5678', 'CBC', False]])
+     ['E5', 'S5678', 'CBC', False],
+     ['E8', 'S9876', None, True],
+     ['S9876', 'E8', None, True],
+     # Negative FARs
+     ['E8', 'S9999', 'Burst', False],
+     ['E9', 'S9999', 'Burst', False],
+     ['E3', 'S9999', 'Burst', False]])
 @patch('gwcelery.tasks.gracedb.get_labels', side_effect=_mock_get_labels)
-@patch('gwcelery.tasks.gracedb.update_superevent')
+@patch('gwcelery.tasks.gracedb.upload.run')
+@patch('gwcelery.tasks.raven.update_coinc_far.run',
+       side_effect=_return_coinc_far_dict)
+@patch('gwcelery.tasks.external_skymaps.plot_overlap_integral.run')
 @patch('gwcelery.tasks.gracedb.create_label.run')
-def test_trigger_raven_alert(mock_create_label, mock_update_superevent,
+def test_trigger_raven_alert(mock_create_label, mock_plot_overlap_integral,
+                             mock_update_coinc_far, mock_upload,
                              mock_get_labels,
                              graceid, result_id, group, expected_result):
     if 'S' in graceid:
@@ -490,27 +658,41 @@ def test_trigger_raven_alert(mock_create_label, mock_update_superevent,
         superevent_id = result_id
         ext_id = graceid
     superevent = _mock_get_event(superevent_id)
+    superevent['labels'] = []
     coinc_far_json = _mock_get_coinc_far(superevent_id)
     ext_event = _mock_get_event(ext_id)
+    ext_event['far'] = 1e-7
+    # Test for additional log message
+    if ext_id == 'E9':
+        ext_event['far'] = 1e-2
     preferred_id = superevent['preferred_event']
+    gw_group = group if group is not None else 'Burst'
     raven.trigger_raven_alert(coinc_far_json, superevent,
-                              graceid, ext_event, group)
+                              graceid, ext_event, gw_group)
 
     if expected_result:
         label_calls = [call('RAVEN_ALERT', superevent_id),
+                       call('HIGH_PROFILE', superevent_id),
                        call('RAVEN_ALERT', ext_id),
                        call('RAVEN_ALERT', preferred_id)]
         mock_create_label.assert_has_calls(label_calls)
+        superevent['labels'] += 'RAVEN_ALERT'
     else:
         mock_create_label.assert_not_called()
 
+    mock_update_coinc_far.assert_called_with(
+        coinc_far_json, superevent, ext_event)
+    mock_plot_overlap_integral.assert_called_with(
+        coinc_far_json, superevent, ext_event)
+    mock_upload.assert_called()
+
 
 @pytest.mark.parametrize(
     'se_id,ext_id,expected_result',
     [['S2468', 'E5', True],
      ['S2468', 'E1', True],
      ['S2468', 'E2', False],
-     ['S2468', 'M5', True],
+     ['S2468', 'M6', True],
      ['S2345', 'E1', False],
      ['S2345', 'E2', False]])
 @patch('gwcelery.tasks.gracedb.create_label.run')
