@@ -1,15 +1,15 @@
 import argparse
 import gzip
-from importlib import resources
 import io
 import os
 from unittest.mock import patch
 
-from astropy.table import Table
 import numpy as np
 import pytest
+from astropy.table import Table
 
-from ..tasks import skymaps
+from ..tasks import external_skymaps, skymaps
+from ..util import read_binary, read_text
 from . import data
 
 
@@ -74,7 +74,7 @@ def test_fits_header(toy_fits_filecontents):
     html = skymaps.fits_header(toy_fits_filecontents, 'test.fits')
 
     # Check output
-    assert html == resources.read_text(data, 'fits_header_result.html')
+    assert html == read_text(data, 'fits_header_result.html')
 
 
 @patch('ligo.skymap.tool.ligo_skymap_plot.main')
@@ -125,6 +125,10 @@ def test_skymap_from_samples(toy_3d_fits_filecontents):
     def mock_skymap_from_samples(args):
         parser = argparse.ArgumentParser()
         parser.add_argument('--outdir', '-o', default='.')
+        parser.add_argument('--instruments', nargs='+')
+        parser.add_argument('--objid', type=str)
+        parser.add_argument('--maxpts', type=int)
+        parser.add_argument('--seed', type=int)
         parser.add_argument('--fitsoutname', default='skymap.fits')
         parser.add_argument('samples')
         parser.add_argument('--jobs', '-j', action='store_true')
@@ -132,21 +136,28 @@ def test_skymap_from_samples(toy_3d_fits_filecontents):
         with open(os.path.join(args.outdir, args.fitsoutname), 'wb') as f:
             f.write(toy_3d_fits_filecontents)
 
-    inbytes = resources.read_binary(data, 'samples.hdf5')
+    inbytes = read_binary(data, 'samples.hdf5')
 
     with patch('ligo.skymap.tool.ligo_skymap_from_samples.main',
                mock_skymap_from_samples):
-        outbytes = skymaps.skymap_from_samples(inbytes)
+        outbytes = skymaps.skymap_from_samples(inbytes, 'S123456ab',
+                                               {'H1', 'L1'})
 
     assert skymaps.is_3d_fits_file(outbytes)
 
 
+@pytest.mark.parametrize(
+    'filename,result',
+    [['test.fits', True],
+     ['test.png', False],
+     # Ensure combined sky maps get blocked
+     [external_skymaps.COMBINED_SKYMAP_FILENAME_MULTIORDER, False]])
 @patch('gwcelery.tasks.gracedb.download.run', mock_download)
 @patch('gwcelery.tasks.gracedb.upload.run')
-def test_handle_plot_coherence(mock_upload):
+def test_handle_plot_coherence(mock_upload, filename, result):
     alert = {
         "data": {
-            "filename": "test.fits",
+            "filename": filename,
             "file_version": 0
         },
         "uid": "T12345",
@@ -154,4 +165,7 @@ def test_handle_plot_coherence(mock_upload):
     }
 
     skymaps.handle_plot_coherence(alert)
-    mock_upload.assert_called_once()
+    if result:
+        mock_upload.assert_called_once()
+    else:
+        mock_upload.assert_not_called()
