@@ -1,15 +1,15 @@
-from celery import exceptions
-import pytest
 from unittest.mock import call, patch
 
+import pytest
+from celery import exceptions
 from requests.exceptions import HTTPError
 from requests.models import Response
 
+from .. import app
 from ..tasks import gracedb, superevents
 from ..util import read_json
 from . import data
 
-
 SUPEREVENTS_NEIGHBOURS = read_json(data, 'superevents.json')
 """Dictionary storing mock superevent neighbors"""
 
@@ -25,6 +25,54 @@ def test_select_preferred_event():
     assert r['graceid'] == 'G3'
 
 
+def test_select_pipeline_preferred_event():
+    """Test selecting preferred event based on pipeline"""
+    sample_event = {
+        'instruments': 'H1,L1',
+        'offline': False,
+        'far': 6e-30,
+        'search': 'AllSky',
+        'labels': ['SKYMAP_READY', 'EMBRIGHT_READY', 'PASTRO_READY'],
+        'extra_attributes': {
+            'CoincInspiral': {
+                'ifos': 'H1,L1',
+                'snr': 30,
+                'combined_far': 6e-30
+            },
+            'SingleInspiral': [{'ifo': 'H1'}, {'ifo': 'L1'}]
+        }
+    }
+    event_snr_combination = {
+        'G1': {'group': 'CBC', 'pipeline': 'gstlal', 'snr': 30},
+        'G2': {'group': 'CBC', 'pipeline': 'gstlal', 'snr': 20},
+        'G3': {'group': 'CBC', 'pipeline': 'pycbc', 'snr': 40},
+        'G4': {'group': 'CBC', 'pipeline': 'pycbc', 'snr': 10},
+        'G5': {'group': 'CBC', 'pipeline': 'spiir', 'snr': 15},
+        'G6': {'group': 'Burst', 'pipeline': 'cwb', 'far': 1e-15},
+        'G7': {'group': 'Burst', 'pipeline': 'cwb', 'far': 1e-30}
+    }
+    events = []
+    for event_id, content in event_snr_combination.items():
+        event = sample_event.copy()
+        event['graceid'] = event_id
+        event['pipeline'] = content['pipeline']
+        event['group'] = content['group']
+        snr = content.get('snr')
+        far = content.get('far')
+        if snr:
+            event['extra_attributes']['CoincInspiral']['snr'] = snr
+        if far:
+            event['far'] = far
+        events.append(event)
+    res = superevents.select_pipeline_preferred_event(events)
+    assert len(res) == 4
+    # expected pipeline preferred events
+    for pipeline, expected_event in {
+            'gstlal': 'G1', 'pycbc': 'G3',
+            'spiir': 'G5', 'cwb': 'G7'}.items():
+        assert res[pipeline]['graceid'] == expected_event
+
+
 @pytest.mark.parametrize(
     'superevent_labels,new_event_labels,preferred_event_labels,new_event_id',
     [[[],
@@ -42,7 +90,7 @@ def test_select_preferred_event():
       ['EMBRIGHT_READY', 'PASTRO_READY'],
       ['EMBRIGHT_READY', 'PASTRO_READY',
        'SKYMAP_READY'], 'T0212'],
-     [['EM_Selected', 'ADVREQ', 'DQOK'],
+     [['LOW_SIGNIF_LOCKED', 'ADVREQ', 'DQOK'],
       ['EMBRIGHT_READY', 'PASTRO_READY',
        'SKYMAP_READY'],
       ['EMBRIGHT_READY', 'PASTRO_READY',
@@ -63,6 +111,7 @@ def test_update_preferred_event(superevent_labels, new_event_labels,
             instruments="I1,J1,K1,L1,M1",
             group="CBC",
             pipeline="gstlal",
+            search="AllSky",
             offline=False,
             superevent="maybe S0039 or empty",
             far=1e-30,
@@ -93,8 +142,8 @@ def test_update_preferred_event(superevent_labels, new_event_labels,
                                        None,
                                        None,
                                        None)
-        # presence of EM_Selected should not
-        if 'EM_Selected' in superevent_labels:
+        # presence of FROZEN_LABEL should not update superevent
+        if superevents.FROZEN_LABEL in superevent_labels:
             p.assert_not_called()
 
         else:
@@ -149,7 +198,7 @@ def test_raven_alert(mock_create_label, labels):
             "pipeline": "gstlal",
             "group": "CBC",
             "search": "AllSky",
-            "far": 1.e-6,
+            "far": 1e-6,
             "instruments": "H1,L1",
             "superevent": "S100",
             "superevent_neighbours": {
@@ -178,7 +227,7 @@ def test_raven_alert(mock_create_label, labels):
                                 'snr': 10.17
                             },
                         },
-                        "far": 1.e-6,
+                        "far": 1e-6,
                         "far_is_upper_limit": False,
                         "gpstime": 1000000,
                         "graceid": "G000003",
@@ -231,8 +280,107 @@ def test_raven_alert(mock_create_label, labels):
         superevents.handle(payload)
     calls = [call('ADVREQ', 'S100')]
     if {'SKYMAP_READY', 'EMBRIGHT_READY', 'PASTRO_READY'}.issubset(labels):
-        calls.append(call('EM_Selected', 'S100'))
-    mock_create_label.assert_has_calls(calls)
+        calls.extend(
+            [call('EM_READY', 'S100'), call('LOW_SIGNIF_LOCKED', 'S100'),
+             call('SIGNIF_LOCKED', 'S100')]
+        )
+    mock_create_label.assert_has_calls(calls, any_order=True)
+
+
+@pytest.mark.parametrize('allsky_far', [1e-6, 1e-9])
+def test_allsky_preferred_over_earlywarning(allsky_far):
+    """New AllSky event (both low-significance and otherwise) updates
+    preferred event of an existing superevent with a low-significance
+    EarlyWarning preferred event."""
+    payload = {
+        "uid": "G123456",
+        "alert_type": "new",
+        "object": {
+            "graceid": "G123456",
+            "pipeline": "gstlal",
+            "group": "CBC",
+            "search": "AllSky",
+            "far": allsky_far,
+            "instruments": "H1,L1",
+            "superevent": None,
+            "superevent_neighbours": {
+                "S100": {  # assumed to be low-significance early-warning
+                    "far": 1e-6,
+                    "gw_events": [
+                        "G567890"
+                    ],
+                    "preferred_event": "G567890",
+                    "preferred_event_data": {
+                        "extra_attributes": {
+                            'SingleInspiral': [
+                                {
+                                    'chisq': 1.07,
+                                    'snr': 10.95,
+                                    'ifo': 'L1'
+                                },
+                                {
+                                    'chisq': 0.54,
+                                    'snr': 12.35,
+                                    'ifo': 'H1'
+                                }
+                            ],
+                            'CoincInspiral': {
+                                'snr': 10.17
+                            },
+                        },
+                        "far": 1.e-6,
+                        "far_is_upper_limit": False,
+                        "gpstime": 1000000,
+                        "graceid": "G567890",
+                        "group": "CBC",
+                        "instruments": "H1,L1",
+                        "labels": ['SKYMAP_READY', 'EMBRIGHT_READY'],
+                        "offline": False,
+                        "pipeline": "gstlal[]",
+                        "reporting_latency": "",
+                        "search": "EarlyWarning",
+                        "submitter": "albert.einstein@LIGO.ORG"
+                    },
+                    "superevent_id": "S100",
+                    "labels": [],
+                    "t_0": 1000000,
+                    "t_end": 1000001,
+                    "t_start": 999999
+                }
+            },
+            "offline": False,
+            "gpstime": 1000000.1,
+            "labels": [],
+            'extra_attributes': {
+                'SingleInspiral': [
+                    {
+                        'chisq': 1.07,
+                        'snr': 8,
+                        'ifo': 'L1'
+                    },
+                    {
+                        'chisq': 0.54,
+                        'snr': 8,
+                        'ifo': 'H1'
+                    }
+                ],
+                'CoincInspiral': {
+                    'snr': 11.3
+                },
+            },
+        },
+    }
+    superevent_response = payload['object']['superevent_neighbours']['S100']
+    with patch('gwcelery.tasks.gracedb.get_superevents',
+               return_value=[superevent_response]), \
+            patch('gwcelery.tasks.gracedb.get_superevent',
+                  return_value=superevent_response), \
+            patch('gwcelery.tasks.gracedb.update_superevent') as p:
+        superevents.handle(payload)
+        p.assert_called_once_with(
+            'S100', t_start=999999, t_end=1000001.1, t_0=1000000.1,
+            preferred_event='G123456'
+        )
 
 
 @pytest.mark.parametrize('labels',
@@ -249,26 +397,56 @@ def test_is_complete(labels):
 
 
 @pytest.mark.parametrize(
-    'labels,label',
-    [[['EMBRIGHT_READY', 'PASTRO_READY'], 'EMBRIGHT_READY'],
-     [['SKYMAP_READY', 'EMBRIGHT_READY', 'PASTRO_READY'], 'SKYMAP_READY']])
-def test_process_called(labels, label):
+    'labels,label,search,alert_type,group,ifos',
+    [[['EMBRIGHT_READY', 'PASTRO_READY'], 'EMBRIGHT_READY', 'AllSky',
+      'label_added', 'CBC', 'H1,L1'],
+     [['SKYMAP_READY', 'EMBRIGHT_READY', 'PASTRO_READY'], 'SKYMAP_READY',
+      'AllSky', 'label_added', 'CBC', 'H1,L1'],
+     [['EMBRIGHT_READY'], 'EMBRIGHT_READY', 'EarlyWarning',
+      'label_added', 'CBC', 'H1,L1'],  # Less-significant EW
+     [['EMBRIGHT_READY'], 'EMBRIGHT_READY', 'EarlyWarning', 'new',
+      'CBC', 'H1,L1,V1'],
+     [[''], '', 'SSM', 'new', 'CBC', 'H1,L1'],
+     [['PASTRO_READY'], '', 'SSM', 'label_added', 'CBC', 'H1,L1'],
+     [[''], '', 'AllSky', 'new', 'CBC', 'H1,L1,K1'],
+     [[''], '', 'AllSky', 'new', 'CBC', 'H1,L1,V1,K1'],
+     [[''], '', 'AllSky', 'new', 'Burst', 'H1,L1,K1']]
+)
+def test_process_called(labels, label, search, alert_type, group, ifos):
     """Test whether the :meth:`superevents.process` is called
     new type IGWN alerts, and label additions that complete the event.
     """
     payload = {
-        "alert_type": "label_added",
+        "alert_type": alert_type,
         "data": {"name": label},
+        "group": group,
+        "instruments": ifos,
         "object": {
             "graceid": "ABCD",
             "far": 1e-6,
-            "group": "CBC",
-            "labels": labels
+            "group": group,
+            "pipeline": "pipeline",
+            "search": search,
+            "labels": labels,
+            "instruments": ifos
         }
     }
+    payload['object']["extra_attributes"] = {
+        "CoincInspiral": {'snr': 4.0},
+        "SingleInspiral": [
+            {"ifo": ifo} for ifo in
+            ifos.split(',')]
+    } if group.lower() == 'cbc' else {"burst_attributes": "burst_values"}
+
     with patch('gwcelery.tasks.superevents.process.run') as process:
         superevents.handle(payload)
-        if superevents.is_complete(payload['object']):
+        if "K1" in ifos:
+            process.assert_not_called()
+        elif search == superevents.VT_SEARCH_NAME:
+            process.assert_not_called()
+        elif superevents.is_complete(payload['object']):
+            process.assert_called()
+        elif alert_type == 'new':
             process.assert_called()
         else:
             process.assert_not_called()
@@ -279,6 +457,7 @@ _mock_event_data = {
     "gpstime": 100.0,
     "pipeline": "gstlal",
     "group": "CBC",
+    "search": "AllSky",
     "far": 1.e-31,
     "instruments": "H1,L1",
     "labels": [],
@@ -336,32 +515,63 @@ def test_upload_same_event():
 
 
 @pytest.mark.parametrize(
-    'group,pipeline,offline,far,instruments,labels,expected_result',
-    [['CBC', 'gstlal', False, 1.e-10, 'H1',
-        ['PASTRO_READY', 'SKYMAP_READY', 'EMBRIGHT_READY'], True],
-     ['CBC', 'gstlal', False, 1.e-10, 'H1',
-         ['PASTRO_READY', 'SKYMAP_READY'], False],
-     ['CBC', 'gstlal', False, 1.e-6, 'H1,L1', ['EMBRIGHT_READY'], False],
-     ['Burst', 'cwb', False, 1e-15, 'H1,L1', ['SKYMAP_READY'], True],
-     ['Burst', 'cwb', False, 1e-15, 'H1,L1', [], False],
-     ['Burst', 'cwb', True, 1e-30, 'H1,L1,V1', [], False],
-     ['CBC', 'gstlal', False, 1.e-6, 'H1,L1,V1', ['RAVEN_ALERT'], False],
-     ['CBC', 'gstlal', False, 1.e-6, 'H1,L1,V1', [
-      'PASTRO_READY', 'SKYMAP_READY', 'EMBRIGHT_READY', 'RAVEN_ALERT'], True],
-     ['CBC', 'gstlal', False, 1.e-15, 'H1,L1,V1', [
-      'PASTRO_READY', 'SKYMAP_READY', 'EMBRIGHT_READY', 'RAVEN_ALERT'],
-      True]])
-def test_should_publish(group, pipeline, offline, far, instruments, labels,
-                        expected_result):
+    'group,pipeline,search,offline,far,instruments,labels,expected_result,'
+    'expected_result_less_significant',
+    [['CBC', 'gstlal', 'AllSky', False, 1.e-10, 'H1',  # complete significant
+        ['PASTRO_READY', 'SKYMAP_READY', 'EMBRIGHT_READY'], True, True],
+     ['CBC', 'gstlal', 'AllSky', False, 1.e-6, 'H1',  # complete less-signif
+        ['PASTRO_READY', 'SKYMAP_READY', 'EMBRIGHT_READY'], False, True],
+     ['CBC', 'gstlal', 'AllSky', False, 2.5e-5, 'H1',  # complete noise
+        ['PASTRO_READY', 'SKYMAP_READY', 'EMBRIGHT_READY'], False, False],
+     ['CBC', 'gstlal', 'AllSky', False, 1.e-6, 'H1',  # incomplete less-signif
+        ['PASTRO_READY', 'SKYMAP_READY'], False, False],
+     ['CBC', 'gstlal', 'AllSky', False, 1.e-6, 'H1,L1',  # incomplete CBC
+        ['EMBRIGHT_READY'], False, False],
+     ['Burst', 'cwb', 'AllSky', False, 1e-15, 'H1,L1',  # complete Burst
+        ['SKYMAP_READY'], True, True],
+     ['Burst', 'cwb', 'AllSky', False, 1e-15, 'H1,L1',  # incomplete Burst
+        [], False, False],
+     ['Burst', 'cwb', 'BBH', False, 1e-15, 'H1,L1',  # complete cWB-BBH
+        ['SKYMAP_READY', 'PASTRO_READY', 'EMBRIGHT_READY'], True, True],
+     ['Burst', 'cwb', 'BBH', False, 1e-15, 'H1,L1',  # incomplete cWB-BBH
+        ['SKYMAP_READY', 'PASTRO_READY'], False, False],
+     ['Burst', 'cwb', 'BBH', False, 1e-15, 'H1,L1',  # incomplete cWB-BBH
+        ['SKYMAP_READY', 'EMBRIGHT_READY'], False, False],
+     ['Burst', 'cwb', 'BBH', False, 1e-15, 'H1,L1',  # incomplete cWB-BBH
+        ['SKYMAP_READY'], False, False],
+     ['Burst', 'cwb', 'BBH', False, 1e-15, 'H1,L1',  # incomplete cWB-BBH
+        [], False, False],
+     ['Burst', 'cwb', 'BBH', True, 1e-30, 'H1,L1,V1',  # incomplete cWB-BBH
+        [], False, False],
+     ['CBC', 'gstlal', 'AllSky', False, 1.e-6, 'H1,L1,V1',  # incomplete ext
+        ['RAVEN_ALERT'], False, False],
+     ['CBC', 'gstlal', 'AllSky', False, 1.e-6, 'H1,L1,V1',  # complete ext
+        ['PASTRO_READY', 'SKYMAP_READY', 'EMBRIGHT_READY', 'RAVEN_ALERT'],
+        True, True],
+     ['CBC', 'gstlal', 'AllSky', False, 1.e-15, 'H1,L1,V1',  # complete 3 ifo significant  # noqa: E501
+        ['PASTRO_READY', 'SKYMAP_READY', 'EMBRIGHT_READY', 'RAVEN_ALERT'],
+        True, True],
+     ['CBC', 'gstlal', 'EarlyWarning', False, 1.e-15, 'H1,L1,V1',  # EW search
+        ['PASTRO_READY', 'SKYMAP_READY', 'EMBRIGHT_READY'],
+        True, True]])
+def test_should_publish(group, pipeline, search, offline, far, instruments,
+                        labels, expected_result,
+                        expected_result_less_significant):
+    """Test publishability criteria under different states of a superevent."""
     event = dict(graceid='G123456',
+                 search=search,
                  group=group,
                  pipeline=pipeline,
                  labels=labels,
                  far=far,
                  offline=offline,
                  instruments=instruments)
+    result_less_significant = (
+        superevents.is_complete(event) and
+        superevents.should_publish(event, significant=False))
     result = (superevents.is_complete(event) and
               superevents.should_publish(event))
+    assert result_less_significant == expected_result_less_significant
     assert result == expected_result
 
 
@@ -387,10 +597,14 @@ def test_raising_http_error(failing_create_superevent):
             "gpstime": 100.0,
             "pipeline": "gstlal",
             "group": "CBC",
+            "search": "AllSky",
             "far": 1.e-31,
             "instruments": "H1,L1",
             "extra_attributes": {
-                "CoincInspiral": {"snr": 20}
+                "CoincInspiral": {"snr": 20},
+                "SingleInspiral": [
+                    {'ifo': ifo} for ifo in ['H1', 'L1']
+                ]
             },
             "offline": False,
             "labels": [],
@@ -430,6 +644,7 @@ def test_parse_trigger_cbc_1():
                         'gpstime': 1163905224.4332082,
                         'group': 'CBC',
                         'pipeline': 'gstlal',
+                        'search': 'AllSky',
                         'offline': False,
                         'far': 3e-09,
                         'instruments': 'H1,L1',
@@ -467,6 +682,7 @@ def test_parse_trigger_cbc_2():
                         'gpstime': 1163905224.4332082,
                         'group': 'CBC',
                         'pipeline': 'gstlal',
+                        'search': 'AllSky',
                         'offline': False,
                         'far': 3e-31,
                         'instruments': 'H1,L1',
@@ -498,11 +714,12 @@ def test_parse_trigger_cbc_2():
         p2.assert_called_once_with('S0039', preferred_event='G000003',
                                    t_0=1163905224.4332082)
         p3.assert_called_once()
-        create_label.assert_called_once_with('ADVREQ', 'S0039')
+        expected_calls = [call('ADVREQ', 'S0039')]
         if superevents.is_complete(event_dictionary):
-            create_label.assert_called_once_with('EM_READY', 'S0039')
-        else:
-            assert_not_called_with(create_label, 'EM_READY', 'S0039')
+            expected_calls.extend(
+                [call('EM_READY', 'S0039'), call('LOW_SIGNIF_LOCKED', 'S0039'),
+                 call('SIGNIF_LOCKED', 'S0039')])
+        create_label.assert_has_calls(expected_calls, any_order=True)
 
 
 def test_parse_trigger_cbc_3():
@@ -513,6 +730,7 @@ def test_parse_trigger_cbc_3():
                         'gpstime': 1286741861.52678,
                         'group': 'CBC',
                         'pipeline': 'gstlal',
+                        'search': 'AllSky',
                         'offline': False,
                         'far': 3e-31,
                         'instruments': 'H1,L1,V1',
@@ -545,6 +763,7 @@ def test_parse_trigger_cbc_4():
                         'gpstime': 1286741861.52678,
                         'group': 'CBC',
                         'pipeline': 'gstlal',
+                        'search': 'AllSky',
                         'offline': False,
                         'far': 5.5e-02,
                         'instruments': 'H1,L1,V1',
@@ -567,6 +786,46 @@ def test_parse_trigger_cbc_4():
         mock_process.assert_not_called()
 
 
+def test_parse_trigger_cbc_5():
+    """New trigger G000002 is complete, with no intersecting superevent,
+    passes less-significant far, but not significant far. New superevent
+    should be created, and labeled LOW_SIGNIF_LOCKED."""
+    event_dictionary = {'graceid': 'G000002',
+                        'gpstime': 1286741861.52678,
+                        'group': 'CBC',
+                        'pipeline': 'gstlal',
+                        'search': 'AllSky',
+                        'offline': False,
+                        'far': 5.5e-07,
+                        'instruments': 'H1,L1,V1',
+                        'labels': [
+                            'PASTRO_READY',
+                            'SKYMAP_READY',
+                            'EMBRIGHT_READY'
+                        ],
+                        'superevent': None,
+                        'superevent_neighbours': SUPEREVENTS_NEIGHBOURS,
+                        'extra_attributes': {
+                            'CoincInspiral': {'snr': 4.0},
+                            'SingleInspiral': [
+                                {"ifo": ifo} for ifo in
+                                ["H1", "L1", "V1"]]}}
+    payload = dict(alert_type='new',
+                   object=event_dictionary,
+                   data=event_dictionary,
+                   uid='G000002')
+    superevents.handle(payload)
+    with patch('gwcelery.tasks.gracedb.create_superevent',
+               return_value='S123456') as p1, \
+            patch('gwcelery.tasks.gracedb.get_superevents',
+                  return_value=SUPEREVENTS_NEIGHBOURS.values()) as p2, \
+            patch('gwcelery.tasks.gracedb.create_label') as p3:
+        superevents.handle(payload)
+        p1.assert_called_once()
+        p2.assert_called_once()
+        p3.assert_called_once_with('LOW_SIGNIF_LOCKED', 'S123456')
+
+
 def test_parse_trigger_burst_1():
     """New cwb trigger G000005 with gpstime lying partially in
     S0039 window, not more significant than existing preferred
@@ -576,6 +835,7 @@ def test_parse_trigger_burst_1():
                         'gpstime': 1163905214.4,
                         'group': 'Burst',
                         'pipeline': 'cwb',
+                        'search': 'AllSky',
                         'offline': False,
                         'far': 3.02e-09,
                         'instruments': 'H1,L1',
@@ -622,6 +882,7 @@ def test_parse_trigger_burst_2():
                         'gpstime': 1163905239.5,
                         'group': 'Burst',
                         'pipeline': 'oLIB',
+                        'search': 'AllSky',
                         'offline': False,
                         'far': 3.02e-16,
                         'instruments': 'H1,L1',
@@ -663,6 +924,7 @@ def test_parse_trigger_burst_3():
                         'gpstime': 1163905249.5,
                         'group': 'Burst',
                         'pipeline': 'oLIB',
+                        'search': 'AllSky',
                         'offline': False,
                         'far': 3.02e-16,
                         'instruments': 'H1,L1',
@@ -697,6 +959,7 @@ def test_parse_trigger_burst_4():
                         'gpstime': 1128658942.9878,
                         'group': 'Burst',
                         'pipeline': 'CWB',
+                        'search': 'AllSky',
                         'offline': False,
                         'far': 1.23e-09,
                         'instruments': 'H1,L1',
@@ -719,6 +982,69 @@ def test_parse_trigger_burst_4():
         p.assert_called_once()
 
 
+def test_parse_trigger_burst_5():
+    """New less-significant oLIB trigger, which is complete, no intersecting
+    superevent. New superevent created and labeled LOW_SIGNIF_LOCKED.
+    """
+    event_dictionary = {'graceid': 'G000007',
+                        'gpstime': 1.0,
+                        'group': 'Burst',
+                        'pipeline': 'oLIB',
+                        'search': 'AllSky',
+                        'offline': False,
+                        'far': 1e-6,
+                        'instruments': 'H1,L1',
+                        'labels': ['SKYMAP_READY'],
+                        'superevent': None,
+                        'superevent_neighbours': SUPEREVENTS_NEIGHBOURS,
+                        'extra_attributes': {
+                            'LalInferenceBurst': {
+                                'quality_mean': 100.0,
+                                'frequency_mean': 100.0,
+                                'omicron_snr_network': 8.0}}}
+    payload = dict(object=event_dictionary,
+                   data=event_dictionary,
+                   alert_type='new',
+                   uid='G000007')
+    with patch('gwcelery.tasks.gracedb.create_superevent',
+               return_value='S123456') as p1, \
+            patch('gwcelery.tasks.gracedb.create_label') as p2:
+        superevents.handle(payload)
+        p1.assert_called_once()
+        p2.assert_called_once_with('LOW_SIGNIF_LOCKED', 'S123456')
+
+
+def test_parse_trigger_burst_6():
+    """New less-significant MLy trigger, which is complete, no intersecting
+    superevent. New superevent created and labeled LOW_SIGNIF_LOCKED.
+    """
+    event_dictionary = {'graceid': 'G123456',
+                        'gpstime': 1.0,
+                        'group': 'Burst',
+                        'pipeline': 'MLy',
+                        'search': 'AllSky',
+                        'offline': False,
+                        'far': 1e-6,
+                        'instruments': 'H1,L1',
+                        'labels': ['SKYMAP_READY'],
+                        'superevent': None,
+                        'superevent_neighbours': SUPEREVENTS_NEIGHBOURS,
+                        'extra_attributes': {
+                            'MLyBurst': {
+                                'duration': 5.0,
+                                'SNR': 100.0}}}
+    payload = dict(object=event_dictionary,
+                   data=event_dictionary,
+                   alert_type='new',
+                   uid='G123456')
+    with patch('gwcelery.tasks.gracedb.create_superevent',
+               return_value='S123456') as p1, \
+            patch('gwcelery.tasks.gracedb.create_label') as p2:
+        superevents.handle(payload)
+        p1.assert_called_once()
+        p2.assert_called_once_with('LOW_SIGNIF_LOCKED', 'S123456')
+
+
 def test_S190421ar_spiir_scenario():    # noqa: N802
     """Test to ensure that a low FAR event with accidental high
     SNR is not promoted to the preferred event status. For example, here,
@@ -732,6 +1058,7 @@ def test_S190421ar_spiir_scenario():    # noqa: N802
                         'group': 'CBC',
                         'instruments': 'H1,L1',
                         'pipeline': 'spiir',
+                        'search': 'AllSky',
                         'offline': False,
                         'labels': [],
                         'superevent': None,
@@ -767,6 +1094,7 @@ def test_inj_means_should_not_publish():
                         'group': 'CBC',
                         'instruments': 'H1,L1',
                         'pipeline': 'spiir',
+                        'search': 'AllSky',
                         'offline': False,
                         'labels': ['INJ'],
                         'extra_attributes': {
@@ -775,3 +1103,38 @@ def test_inj_means_should_not_publish():
                             'SingleInspiral': [
                                 {"ifo": ifo} for ifo in ["H1", "L1"]]}}
     assert superevents.should_publish(event_dictionary) is False
+
+
+@pytest.fixture
+def burst_bbh_ranked_higher_than_burst_allsky(monkeypatch):
+    monkeypatch.setitem(
+        app.conf, 'superevent_candidate_preference',
+        {'burst': {'bbh': 1, 'allsky': 0}})
+
+
+def test_burst_bbh_vs_allsky(burst_bbh_ranked_higher_than_burst_allsky):
+    """Test event ranking when different searches under the same group
+    is ranked differently. In this case the burst BBH search is ranked higher
+    than burst AllSky."""
+    event_dictionary = {
+        'graceid': 'G1',
+        'gpstime': 1.0,
+        'pipeline': 'CWB',
+        'offline': False,
+        'far': 1e-10,
+        'instruments': 'H1,L1',
+        'labels': [],
+        'superevent': None,
+        'superevent_neighbours': SUPEREVENTS_NEIGHBOURS,
+        'extra_attributes': {
+            'MultiBurst': {
+                'duration': 0.02,
+                'start_time': 0.9,
+                'snr': 19.0
+            }
+        }
+    }
+
+    event_1 = dict(**event_dictionary, group='Burst', search='AllSky')
+    event_2 = dict(**event_dictionary, group='Burst', search='BBH')
+    assert superevents.keyfunc(event_1) < superevents.keyfunc(event_2)
