@@ -1,7 +1,10 @@
 from distutils.spawn import find_executable
+from pathlib import Path
 from unittest.mock import Mock
 
+import lal
 import pytest
+from gwpy.time import tconvert
 
 from .. import app, main
 from ..tools import nagios
@@ -25,8 +28,9 @@ def test_nagios_unknown_error(monkeypatch, capsys):
 def celery_worker_parameters():
     return dict(
         perform_ping_check=False,
-        queues=['celery', 'exttrig', 'kafka', 'openmp', 'superevent',
-                'voevent']
+        queues=['celery', 'exttrig', 'em-bright', 'kafka-consumer',
+                'kafka-producer', 'multiprocessing', 'openmp', 'superevent',
+                'voevent', 'skynet']
     )
 
 
@@ -36,16 +40,37 @@ def test_nagios(capsys, monkeypatch, request, socket_enabled, starter,
     mock_hop_stream_object = Mock()
     mock_hop_stream_object.configure_mock(**{'close.return_value': None})
     mock_hop_stream = Mock(return_value=mock_hop_stream_object)
+    mock_get_auth = Mock()
     mock_list_topics = Mock()
     unix_socket = str(tmp_path / 'redis.sock')
     broker_url = f'redis+socket://{unix_socket}'
+    orig_llhoft_path = app.conf['llhoft_glob']
+    llhoft_path = str(
+        Path(__file__).parent / 'data/llhoft/fail/{detector}/*.gwf')
 
     monkeypatch.setattr('hop.io.Stream.open', mock_hop_stream)
     monkeypatch.setattr('igwn_alert.client', mock_igwn_alert_client)
+    monkeypatch.setattr('gwcelery.kafka.bootsteps.KafkaBase.get_auth',
+                        mock_get_auth)
     monkeypatch.setattr('gwcelery.kafka.bootsteps.list_topics',
                         mock_list_topics)
     monkeypatch.setitem(app.conf, 'broker_url', broker_url)
     monkeypatch.setitem(app.conf, 'result_backend', broker_url)
+    monkeypatch.setitem(app.conf, 'kafka_consumer_config', {
+        'swift': {
+            'url': 'kafka://kafka.gcn.nasa.gov/swift.bat.guano',
+            'suffix': 'json'
+        }
+    })
+    monkeypatch.setitem(app.conf, 'llhoft_glob', llhoft_path)
+
+    # no llhoft distributed in last 10 mins
+    with pytest.raises(SystemExit) as excinfo:
+        main(['gwcelery', 'nagios'])
+    assert excinfo.value.code == nagios.NagiosPluginStatus.CRITICAL
+    out, err = capsys.readouterr()
+    assert 'CRITICAL: Low-latency hoft is not being streamed' in out
+    monkeypatch.setitem(app.conf, 'llhoft_glob', orig_llhoft_path)
 
     # no broker
 
@@ -142,6 +167,26 @@ def test_nagios(capsys, monkeypatch, request, socket_enabled, starter,
         'gwcelery.tools.nagios.get_celery_queue_length',
         Mock(return_value=0))
 
+    # no superevents in last hour
+    monkeypatch.setattr(
+        'gwcelery.tools.nagios.get_recent_mdc_superevents',
+        Mock(return_value=(
+            [{'superevent_id': 'MS123456',
+              'created': str(tconvert(lal.GPSTimeNow() - 7200))}],
+            lal.GPSTimeNow() - 6 * 3600, lal.GPSTimeNow())))
+    with pytest.raises(SystemExit) as excinfo:
+        main(['gwcelery', 'nagios'])
+    assert excinfo.value.code == nagios.NagiosPluginStatus.CRITICAL
+    out, err = capsys.readouterr()
+    assert 'CRITICAL: No MDC superevents found in last one hour' in out
+    # reset to expected behavior for later checks
+    monkeypatch.setattr(
+        'gwcelery.tools.nagios.get_recent_mdc_superevents',
+        Mock(return_value=(
+            [{'superevent_id': 'MS123456',
+              'created': str(tconvert(lal.GPSTimeNow() - 100))}],
+            lal.GPSTimeNow() - 6 * 3600, lal.GPSTimeNow())))
+
     # Kafka broker, topic or broker are down
     monkeypatch.setattr(
         'celery.app.control.Inspect.stats',
@@ -189,7 +234,32 @@ def test_nagios(capsys, monkeypatch, request, socket_enabled, starter,
                                     True},
                                    'kafka_delivery_failures':
                                    {'kafka://kafka.scimma.org/gwalert-test':
-                                    False}}}))  # noqa: E501
+                                    False},
+                                   'active_kafka_consumers': set()}}))
+
+    with pytest.raises(SystemExit) as excinfo:
+        main(['gwcelery', 'nagios'])
+    assert excinfo.value.code == nagios.NagiosPluginStatus.CRITICAL
+    out, err = capsys.readouterr()
+    assert 'CRITICAL: Not all Kafka consumer bootstep topics are active' \
+           in out
+
+    expected_kafka_consumer_topics = \
+        nagios.get_expected_kafka_consumer_bootstep_names(app)
+    monkeypatch.setattr(
+        'celery.app.control.Inspect.stats',
+        Mock(return_value={'foo': {'voevent-broker-peers': ['127.0.0.1'],
+                                   'voevent-receiver-peers': ['127.0.0.1'],
+                                   'igwn-alert-topics':
+                                   expected_igwn_alert_topics,
+                                   'kafka_topic_up':
+                                   {'kafka://kafka.scimma.org/gwalert-test':
+                                    True},
+                                   'kafka_delivery_failures':
+                                   {'kafka://kafka.scimma.org/gwalert-test':
+                                    False},
+                                   'active_kafka_consumers':
+                                   expected_kafka_consumer_topics}}))
 
     with pytest.raises(SystemExit) as excinfo:
         main(['gwcelery', 'nagios'])
