@@ -2,9 +2,9 @@
 :class:`comet.protocol.VOEventSubscriberFactory` that allow inspection of the
 list of active subscribers.
 """
-from comet.protocol.subscriber import (
-    VOEventSubscriber as _VOEventSubscriber,
-    VOEventSubscriberFactory as _VOEventSubscriberFactory)
+from comet.protocol.subscriber import VOEventSubscriber as _VOEventSubscriber
+from comet.protocol.subscriber import \
+    VOEventSubscriberFactory as _VOEventSubscriberFactory
 
 
 class VOEventSubscriber(_VOEventSubscriber):
